@@ -95,19 +95,19 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  if(IsFakeNode(finish_node))
     GetFakeLatLong(finish_node,&finish_lat,&finish_lon);
  else
-    GetLatLong(nodes,finish_node,&finish_lat,&finish_lon);
+    GetLatLong(nodes,finish_node,NULL,&finish_lat,&finish_lon);
 
  /* Create the list of results and insert the first node into the queue */
 
- results=NewResultsList(64);
- queue=NewQueueList();
+ results=NewResultsList(8);
+ queue=NewQueueList(8);
 
  results->start_node=start_node;
  results->prev_segment=prev_segment;
 
  result1=InsertResult(results,results->start_node,results->prev_segment);
 
- InsertInQueue(queue,result1);
+ InsertInQueue(queue,result1,0);
 
  /* Check for barrier at start waypoint - must perform U-turn */
 
@@ -278,10 +278,7 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           finish_result=result2;
          }
        else
-         {
-          result2->sortby=result2->score;
-          InsertInQueue(queue,result2);
-         }
+          InsertInQueue(queue,result2,result2->score);
 
       endloop:
 
@@ -377,12 +374,12 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
  if(IsFakeNode(end->finish_node))
     GetFakeLatLong(end->finish_node,&finish_lat,&finish_lon);
  else
-    GetLatLong(nodes,end->finish_node,&finish_lat,&finish_lon);
+    GetLatLong(nodes,end->finish_node,NULL,&finish_lat,&finish_lon);
 
  /* Create the list of results and insert the first node into the queue */
 
- results=NewResultsList(65536);
- queue=NewQueueList();
+ results=NewResultsList(20);
+ queue=NewQueueList(12);
 
  results->start_node=begin->start_node;
  results->prev_segment=begin->prev_segment;
@@ -427,9 +424,8 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
           result2->prev=result5;
 
           result2->score=result3->score;
-          result2->sortby=result3->score;
 
-          InsertInQueue(queue,result2);
+          InsertInQueue(queue,result2,result3->score);
 
           if((result4=FindResult(end,result2->node,result2->segment)))
             {
@@ -446,7 +442,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
    }
 
  if(begin->number==1)
-    InsertInQueue(queue,result1);
+    InsertInQueue(queue,result1,0);
 
  /* Check for barrier at start waypoint - must perform U-turn */
 
@@ -599,18 +595,19 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
          {
           double lat,lon;
           distance_t direct;
+          score_t potential_score;
 
-          GetLatLong(nodes,node2,&lat,&lon); /* node2 cannot be a fake node (must be a super-node) */
+          GetLatLong(nodes,node2,node2p,&lat,&lon); /* node2 cannot be a fake node (must be a super-node) */
 
           direct=Distance(lat,lon,finish_lat,finish_lon);
 
           if(option_quickest==0)
-             result2->sortby=result2->score+(score_t)direct/profile->max_pref;
+             potential_score=result2->score+(score_t)direct/profile->max_pref;
           else
-             result2->sortby=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
+             potential_score=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
 
-          if(result2->sortby<finish_score)
-             InsertInQueue(queue,result2);
+          if(potential_score<finish_score)
+             InsertInQueue(queue,result2,potential_score);
          }
 
 #if !DEBUG
@@ -773,15 +770,15 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
 
  /* Create the list of results and insert the first node into the queue */
 
- results=NewResultsList(64);
- queue=NewQueueList();
+ results=NewResultsList(8);
+ queue=NewQueueList(8);
 
  results->start_node=start_node;
  results->prev_segment=NO_SEGMENT;
 
  result1=InsertResult(results,results->start_node,results->prev_segment);
 
- InsertInQueue(queue,result1);
+ InsertInQueue(queue,result1,0);
 
  /* Loop across all nodes in the queue */
 
@@ -838,20 +835,18 @@ static Results *FindSuperRoute(Nodes *nodes,Segments *segments,Ways *ways,Relati
           result2=InsertResult(results,node2,seg2);
           result2->prev=result1;
           result2->score=cumulative_score;
-          result2->sortby=result2->score;
          }
        else if(cumulative_score<result2->score) /* New score for end node/segment combination is better */
          {
           result2->prev=result1;
           result2->segment=seg2;
           result2->score=cumulative_score;
-          result2->sortby=result2->score;
          }
        else goto endloop;
 
        /* don't route beyond a super-node. */
        if(!IsSuperNode(node2p))
-          InsertInQueue(queue,result2);
+          InsertInQueue(queue,result2,result2->score);
 
       endloop:
 
@@ -911,15 +906,15 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
  /* Create the list of results and insert the first node into the queue */
 
- results=NewResultsList(64);
- queue=NewQueueList();
+ results=NewResultsList(8);
+ queue=NewQueueList(8);
 
  results->start_node=start_node;
  results->prev_segment=prev_segment;
 
  result1=InsertResult(results,results->start_node,results->prev_segment);
 
- InsertInQueue(queue,result1);
+ InsertInQueue(queue,result1,0);
 
  /* Check for barrier at start waypoint - must perform U-turn */
 
@@ -1068,20 +1063,29 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
           if(node2==finish_node)
              nsuper++;
+
+          if(node2==finish_node)
+            {
+             results->finish_node=finish_node;
+             results->last_segment=seg2;
+            }
          }
        else if(cumulative_score<result2->score) /* New score for end node/segment combination is better */
          {
           result2->prev=result1;
           result2->score=cumulative_score;
+
+          if(node2==finish_node)
+            {
+             results->finish_node=finish_node;
+             results->last_segment=seg2;
+            }
          }
        else
           goto endloop;
 
        if(node2p && !IsSuperNode(node2p))
-         {
-          result2->sortby=result2->score;
-          InsertInQueue(queue,result2);
-         }
+          InsertInQueue(queue,result2,result2->score);
 
       endloop:
 
@@ -1174,7 +1178,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
 
   Profile *profile The profile containing the transport type, speeds and allowed highways.
 
-  Results *being The partial set of routes already computed.
+  Results *begin The partial set of routes already computed.
 
   index_t finish_node The finish node.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -1193,7 +1197,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
 
  /* Check the list of results and insert the super nodes into the queue */
 
- queue=NewQueueList();
+ queue=NewQueueList(8);
 
  result3=FirstResult(begin);
 
@@ -1208,7 +1212,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
 
     if(!IsFakeNode(result3->node))
        if(IsSuperNode(LookupNode(nodes,result3->node,5)))
-          InsertInQueue(queue,result3);
+          InsertInQueue(queue,result3,result3->score);
 
     result3=NextResult(results,result3);
    }
@@ -1363,10 +1367,7 @@ Results *ExtendStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations
             }
          }
        else
-         {
-          result2->sortby=result2->score;
-          InsertInQueue(queue,result2);
-         }
+          InsertInQueue(queue,result2,result2->score);
 
       endloop:
 
@@ -1433,14 +1434,14 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
  /* Create the results and insert the finish node into the queue */
 
- results=NewResultsList(64);
- queue=NewQueueList();
+ results=NewResultsList(8);
+ queue=NewQueueList(8);
 
  results->finish_node=finish_node;
 
  result1=InsertResult(results,finish_node,NO_SEGMENT);
 
- InsertInQueue(queue,result1);
+ InsertInQueue(queue,result1,0);
 
  /* Loop across all nodes in the queue */
 
@@ -1585,10 +1586,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
           goto endloop;
 
        if(IsFakeNode(node1) || !IsSuperNode(node1p))
-         {
-          result2->sortby=result2->score;
-          InsertInQueue(queue,result2);
-         }
+          InsertInQueue(queue,result2,result2->score);
 
       endloop:
 
@@ -1615,7 +1613,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *
 
  /* Create a results structure with the node at the end of the segment opposite the start */
 
- results2=NewResultsList(64);
+ results2=NewResultsList(8);
 
  results2->finish_node=results->finish_node;
 
@@ -1713,7 +1711,7 @@ Results *CombineRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *rel
  printf("  CombineRoutes(...,[begin has %d nodes],[middle has %d nodes])\n",begin->number,middle->number);
 #endif
 
- combined=NewResultsList(256);
+ combined=NewResultsList(10);
 
  combined->start_node=begin->start_node;
  combined->prev_segment=begin->prev_segment;
