@@ -71,7 +71,7 @@ static int64_t node_refid=0,way_refid=0,relation_refid=0;
 
 static int mode_change=MODE_NORMAL;
 
-static int buffer_allocated;
+static uint32_t buffer_allocated;
 static unsigned char *buffer=NULL;
 static unsigned char *buffer_ptr,*buffer_end;
 
@@ -275,12 +275,12 @@ int ParseO5M(int fd,int changes)
 
  /* Print the initial message */
 
- nnodes=0,nways=0,nrelations=0;
-
  printf_first("Reading: Bytes=0 Nodes=0 Ways=0 Relations=0");
 
  /* The actual parser. */
 
+ nnodes=0,nways=0,nrelations=0;
+
  if(changes)
     mode_change=MODE_MODIFY;
 
@@ -424,27 +424,27 @@ int ParseO5M(int fd,int changes)
 
 
    case O5M_ERROR_UNEXP_EOF:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: unexpected end of file seen.\n",byteno);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": unexpected end of file seen.\n",byteno);
     break;
 
    case O5M_ERROR_RESET_NOT_FIRST:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: Reset was not the first byte.\n",byteno);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": Reset was not the first byte.\n",byteno);
     break;
 
    case O5M_ERROR_HEADER_NOT_FIRST:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: Header was not the first section.\n",byteno);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": Header was not the first section.\n",byteno);
     break;
 
    case O5M_ERROR_EXPECTED_O5M:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: Expected O5M format but header disagrees.\n",byteno);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": Expected O5M format but header disagrees.\n",byteno);
     break;
 
    case O5M_ERROR_EXPECTED_O5C:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: Expected O5C format but header disagrees.\n",byteno);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": Expected O5C format but header disagrees.\n",byteno);
     break;
 
    case O5M_ERROR_FILE_LEVEL:
-    fprintf(stderr,"O5M Parser: Error at byte %llu: Unexpected dataset type %02x.\n",byteno,error);
+    fprintf(stderr,"O5M Parser: Error at byte %"PRIu64": Unexpected dataset type %02x.\n",byteno,error);
     break;
    }
 
@@ -786,3 +786,71 @@ static unsigned char *process_string(int pair,unsigned char **buf_ptr,unsigned c
 
  return(string);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Parse an O5M format OSM file (from planet download).
+
+  int ParseO5MFile Returns 0 if OK or something else in case of an error.
+
+  int fd The file descriptor of the file to read from.
+
+  NodesX *OSMNodes The data structure of nodes to fill in.
+
+  WaysX *OSMWays The data structure of ways to fill in.
+
+  RelationsX *OSMRelations The data structure of relations to fill in.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int ParseO5MFile(int fd,NodesX *OSMNodes,WaysX *OSMWays,RelationsX *OSMRelations)
+{
+ int retval;
+
+ /* Initialise the parser */
+
+ InitialiseParser(OSMNodes,OSMWays,OSMRelations);
+
+ /* Parse the file */
+
+ retval=ParseO5M(fd,0);
+
+ /* Cleanup the parser */
+
+ CleanupParser();
+
+ return(retval);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Parse an O5C format OSM file (from planet download).
+
+  int ParseO5CFile Returns 0 if OK or something else in case of an error.
+
+  int fd The file descriptor of the file to read from.
+
+  NodesX *OSMNodes The data structure of nodes to fill in.
+
+  WaysX *OSMWays The data structure of ways to fill in.
+
+  RelationsX *OSMRelations The data structure of relations to fill in.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int ParseO5CFile(int fd,NodesX *OSMNodes,WaysX *OSMWays,RelationsX *OSMRelations)
+{
+ int retval;
+
+ /* Initialise the parser */
+
+ InitialiseParser(OSMNodes,OSMWays,OSMRelations);
+
+ /* Parse the file */
+
+ retval=ParseO5M(fd,1);
+
+ /* Cleanup the parser */
+
+ CleanupParser();
+
+ return(retval);
+}
