@@ -29,7 +29,6 @@
 #include "typesx.h"
 
 #include "nodesx.h"
-#include "segmentsx.h"
 #include "waysx.h"
 #include "relationsx.h"
 
@@ -49,7 +48,6 @@
 /* Local variables */
 
 static NodesX     *nodes;
-static SegmentsX  *segments;
 static WaysX      *ways;
 static RelationsX *relations;
 
@@ -74,29 +72,20 @@ static double parse_length(way_t id,const char *k,const char *v);
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Parse an OSM XML file (from JOSM or planet download).
-
-  int ParseOSMFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor of the file to read from.
+  Initialise the OSM parser by initialising the local variables.
 
   NodesX *OSMNodes The data structure of nodes to fill in.
 
-  SegmentsX *OSMSegments The data structure of segments to fill in.
-
   WaysX *OSMWays The data structure of ways to fill in.
 
   RelationsX *OSMRelations The data structure of relations to fill in.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int ParseOSMFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
+void InitialiseParser(NodesX *OSMNodes,WaysX *OSMWays,RelationsX *OSMRelations)
 {
- int retval;
-
  /* Copy the function parameters and initialise the variables */
 
  nodes=OSMNodes;
- segments=OSMSegments;
  ways=OSMWays;
  relations=OSMRelations;
 
@@ -105,60 +94,15 @@ int ParseOSMFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,R
  relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
  relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
  relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
-
- /* Parse the file */
-
- retval=ParseXML(fd,xml_osm_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
-
- /* Free the variables */
-
- free(way_nodes);
-
- free(relation_nodes);
- free(relation_ways);
- free(relation_relations);
-
- return(retval);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Parse an OSC XML file (from planet download).
-
-  int ParseOSCFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor of the file to read from.
-
-  NodesX *OSMNodes The data structure of nodes to fill in.
-
-  SegmentsX *OSMSegments The data structure of segments to fill in.
-
-  WaysX *OSMWays The data structure of ways to fill in.
-
-  RelationsX *OSMRelations The data structure of relations to fill in.
+  Clean up the memory after parsing.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int ParseOSCFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
+void CleanupParser(void)
 {
- int retval;
-
- /* Copy the function parameters and initialise the variables */
-
- nodes=OSMNodes;
- segments=OSMSegments;
- ways=OSMWays;
- relations=OSMRelations;
-
- way_nodes=(node_t*)malloc(256*sizeof(node_t));
-
- relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
- relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
- relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
-
- /* Parse the file */
-
- retval=ParseXML(fd,xml_osc_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
-
  /* Free the variables */
 
  free(way_nodes);
@@ -166,155 +110,6 @@ int ParseOSCFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,R
  free(relation_nodes);
  free(relation_ways);
  free(relation_relations);
-
- return(retval);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Parse a PBF format OSM file (from planet download).
-
-  int ParsePBFFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor of the file to read from.
-
-  NodesX *OSMNodes The data structure of nodes to fill in.
-
-  SegmentsX *OSMSegments The data structure of segments to fill in.
-
-  WaysX *OSMWays The data structure of ways to fill in.
-
-  RelationsX *OSMRelations The data structure of relations to fill in.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int ParsePBFFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
-{
- int retval;
-
- /* Copy the function parameters and initialise the variables */
-
- nodes=OSMNodes;
- segments=OSMSegments;
- ways=OSMWays;
- relations=OSMRelations;
-
- way_nodes=(node_t*)malloc(256*sizeof(node_t));
-
- relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
- relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
- relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
-
- /* Parse the file */
-
- retval=ParsePBF(fd);
-
- /* Free the variables */
-
- free(way_nodes);
-
- free(relation_nodes);
- free(relation_ways);
- free(relation_relations);
-
- return(retval);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Parse an O5M format OSM file (from planet download).
-
-  int ParseO5MFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor of the file to read from.
-
-  NodesX *OSMNodes The data structure of nodes to fill in.
-
-  SegmentsX *OSMSegments The data structure of segments to fill in.
-
-  WaysX *OSMWays The data structure of ways to fill in.
-
-  RelationsX *OSMRelations The data structure of relations to fill in.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int ParseO5MFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
-{
- int retval;
-
- /* Copy the function parameters and initialise the variables */
-
- nodes=OSMNodes;
- segments=OSMSegments;
- ways=OSMWays;
- relations=OSMRelations;
-
- way_nodes=(node_t*)malloc(256*sizeof(node_t));
-
- relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
- relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
- relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
-
- /* Parse the file */
-
- retval=ParseO5M(fd,0);
-
- /* Free the variables */
-
- free(way_nodes);
-
- free(relation_nodes);
- free(relation_ways);
- free(relation_relations);
-
- return(retval);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Parse an O5C format OSM file (from planet download).
-
-  int ParseO5CFile Returns 0 if OK or something else in case of an error.
-
-  int fd The file descriptor of the file to read from.
-
-  NodesX *OSMNodes The data structure of nodes to fill in.
-
-  SegmentsX *OSMSegments The data structure of segments to fill in.
-
-  WaysX *OSMWays The data structure of ways to fill in.
-
-  RelationsX *OSMRelations The data structure of relations to fill in.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int ParseO5CFile(int fd,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
-{
- int retval;
-
- /* Copy the function parameters and initialise the variables */
-
- nodes=OSMNodes;
- segments=OSMSegments;
- ways=OSMWays;
- relations=OSMRelations;
-
- way_nodes=(node_t*)malloc(256*sizeof(node_t));
-
- relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
- relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
- relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
-
- /* Parse the file */
-
- retval=ParseO5M(fd,1);
-
- /* Free the variables */
-
- free(way_nodes);
-
- free(relation_nodes);
- free(relation_ways);
- free(relation_relations);
-
- return(retval);
 }
 
 
@@ -473,7 +268,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Bicycle;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'bicycle' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'bicycle' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -485,7 +280,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Foot;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'foot' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'foot' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -497,7 +292,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Goods;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'goods' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'goods' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -509,7 +304,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Horse;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'horse' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'horse' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -518,7 +313,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_HGV;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'hgv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'hgv' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -530,7 +325,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Moped;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'moped' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'moped' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -539,7 +334,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Motorcycle;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'motorcycle' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'motorcycle' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -548,7 +343,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Motorcar;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'motorcar' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'motorcar' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -560,7 +355,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_PSV;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'psv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'psv' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -584,7 +379,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
           if(ISFALSE(v))
              allow&=~Transports_Wheelchair;
           else if(!ISTRUE(v))
-             logerror("Node %"Pnode_t" has an unrecognised tag 'wheelchair' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+             logerror("Node %"Pnode_t" has an unrecognised tag 'wheelchair' = '%s' (after tagging rules); using 'yes'.\n",logerror_node(id),v);
           recognised=1; break;
          }
 
@@ -595,7 +390,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
       }
 
     if(!recognised)
-       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_node(id),k,v);
    }
 
  /* Create the node */
@@ -617,11 +412,11 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
 void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
 {
  Way way={0};
- distance_t oneway=0,area=0;
+ int oneway=0,area=0;
  int roundabout=0,lanes=0;
  char *name=NULL,*ref=NULL,*refname=NULL;
  way_t id;
- int i,j;
+ int i;
 
  /* Convert id */
 
@@ -634,11 +429,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
    {
     way.type=WAY_DELETED;
 
-    AppendWayList(ways,id,&way,"");
-
-    way.type=Highway_None;
-
-    AppendSegmentList(segments,id,NO_NODE_ID,NO_NODE_ID,0);
+    AppendWayList(ways,id,&way,way_nodes,way_nnodes,"");
    }
 
  if(mode==MODE_DELETE)
@@ -648,13 +439,14 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
 
  if(way_nnodes==0)
    {
-    logerror("Way %"Pway_t" has no nodes.\n",id);
+    logerror("Way %"Pway_t" has no nodes.\n",logerror_way(id));
     return;
    }
 
  if(way_nnodes==1)
    {
-    logerror("Way %"Pway_t" has only one node.\n",id);
+    logerror_node(way_nodes[0]); /* Extra logerror information since way isn't stored */
+    logerror("Way %"Pway_t" has only one node.\n",logerror_way(id));
     return;
    }
 
@@ -670,7 +462,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
        way.type=HighwayType(v);
 
        if(way.type==Highway_None)
-          logerror("Way %"Pway_t" has an unrecognised highway type '%s' (after tagging rules); ignoring it.\n",id,v);
+          logerror("Way %"Pway_t" has an unrecognised highway type '%s' (after tagging rules); ignoring it.\n",logerror_way(id),v);
 
        break;
       }
@@ -695,9 +487,9 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
        if(!strcmp(k,"area"))
          {
           if(ISTRUE(v))
-             area=SEGMENT_AREA;
+             area=1;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'area' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'area' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -709,7 +501,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Bicycle;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'bicycle' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'bicycle' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -718,7 +510,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_BicycleRoute;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -727,7 +519,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_Bridge;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'bridge' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'bridge' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -739,7 +531,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Foot;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'foot' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'foot' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -748,7 +540,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_FootRoute;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'footroute' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -760,7 +552,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Goods;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'goods' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'goods' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -775,7 +567,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Horse;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'horse' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'horse' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -784,7 +576,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_HGV;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'hgv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'hgv' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -798,7 +590,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(sscanf(v,"%f%n",&lanesf,&en)==1 && en && !v[en])
              lanes=(int)lanesf;
           else
-             logerror("Way %"Pway_t" has an unrecognised tag 'lanes' = '%s' (after tagging rules); ignoring it.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'lanes' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -843,7 +635,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Moped;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'moped' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'moped' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -852,7 +644,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Motorcycle;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'motorcycle' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'motorcycle' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -861,7 +653,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Motorcar;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'motorcar' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'motorcar' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -870,7 +662,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_Multilane;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'multilane' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'multilane' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -889,11 +681,11 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
        if(!strcmp(k,"oneway"))
          {
           if(ISTRUE(v))
-             oneway=ONEWAY_1TO2;
+             oneway=1;
           else if(!strcmp(v,"-1"))
-             oneway=ONEWAY_2TO1;
+             oneway=-1;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'oneway' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'oneway' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -905,7 +697,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_Paved;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'paved' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'paved' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -914,7 +706,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_PSV;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'psv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'psv' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -932,7 +724,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              roundabout=1;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'roundabout' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'roundabout' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -944,7 +736,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.props|=Properties_Tunnel;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'tunnel' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'tunnel' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -956,7 +748,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
           if(ISTRUE(v))
              way.allow|=Transports_Wheelchair;
           else if(!ISFALSE(v))
-             logerror("Way %"Pway_t" has an unrecognised tag 'wheelchair' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Way %"Pway_t" has an unrecognised tag 'wheelchair' = '%s' (after tagging rules); using 'no'.\n",logerror_way(id),v);
           recognised=1; break;
          }
 
@@ -967,14 +759,14 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
       }
 
     if(!recognised)
-       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
    }
 
  /* Create the way */
 
  if(area && oneway)
    {
-    logerror("Way %"Pway_t" is an area and oneway; ignoring area tagging.\n",id);
+    logerror("Way %"Pway_t" is an area and oneway; ignoring area tagging.\n",logerror_way(id));
     area=0;
    }
 
@@ -982,11 +774,31 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
     return;
 
  if(oneway)
+   {
     way.type|=Highway_OneWay;
 
+    if(oneway==-1)
+       for(i=0;i<way_nnodes/2;i++)
+         {
+          node_t temp;
+
+          temp=way_nodes[i];
+          way_nodes[i]=way_nodes[way_nnodes-i-1];
+          way_nodes[way_nnodes-i-1]=temp;
+         }
+   }
+
  if(roundabout)
     way.type|=Highway_Roundabout;
 
+ if(area)
+   {
+    way.type|=Highway_Area;
+
+    if(way_nodes[0]!=way_nodes[way_nnodes-1])
+       logerror("Way %"Pway_t" is an area but not closed.\n",logerror_way(id));
+   }
+
  if(lanes)
    {
     if(oneway || (lanes/2)>1)
@@ -1008,47 +820,10 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
  else /* if(!ref && !name) */
     refname="";
 
- AppendWayList(ways,id,&way,refname);
+ AppendWayList(ways,id,&way,way_nodes,way_nnodes,refname);
 
  if(ref && name)
     free(refname);
-
- if(area && way_nodes[0]!=way_nodes[way_nnodes-1])
-    logerror("Way %"Pway_t" is an area but not closed.\n",id);
-
- for(i=1;i<way_nnodes;i++)
-   {
-    node_t from=way_nodes[i-1];
-    node_t to  =way_nodes[i];
-
-    if(from==to)
-       logerror("Node %"Pnode_t" in way %"Pway_t" is connected to itself.\n",from,id);
-    else
-      {
-       int nto=1,duplicated=0;
-
-       for(j=1;j<i;j++)
-         {
-          node_t n1=way_nodes[j-1];
-          node_t n2=way_nodes[j];
-
-          if(n1==to && (i!=way_nnodes-1 || j!=1))
-             nto++;
-
-          if((n1==from && n2==to) || (n2==from && n1==to))
-            {
-             duplicated=1;
-             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" in way %"Pway_t" is duplicated.\n",n1,n2,id);
-            }
-         }
-
-       if(nto>=2 && !duplicated)
-          logerror("Node %"Pnode_t" in way %"Pway_t" appears more than once.\n",to,id);
-
-       if(!duplicated)
-          AppendSegmentList(segments,id,from,to,area+oneway);
-      }
-   }
 }
 
 
@@ -1081,6 +856,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
  if(mode==MODE_DELETE || mode==MODE_MODIFY)
    {
     AppendRouteRelationList(relations,id,RELATION_DELETED,
+                            relation_nodes,relation_nnodes,
                             relation_ways,relation_nways,
                             relation_relations,relation_nrelations);
 
@@ -1096,7 +872,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
 
  if(relation_nnodes==0 && relation_nways==0 && relation_nrelations==0)
    {
-    logerror("Relation %"Prelation_t" has no nodes, ways or relations.\n",id);
+    logerror("Relation %"Prelation_t" has no nodes, ways or relations.\n",logerror_relation(id));
     return;
    }
 
@@ -1116,7 +892,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
           if(ISTRUE(v))
              routes|=Transports_Bicycle;
           else if(!ISFALSE(v))
-             logerror("Relation %"Prelation_t" has an unrecognised tag 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Relation %"Prelation_t" has an unrecognised tag 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",logerror_relation(id),v);
           recognised=1; break;
          }
 
@@ -1130,7 +906,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
                 except|=TRANSPORTS(i);
 
           if(except==Transports_None)
-             logerror("Relation %"Prelation_t" has an unrecognised tag 'except' = '%s' (after tagging rules); ignoring it.\n",id,v);
+             logerror("Relation %"Prelation_t" has an unrecognised tag 'except' = '%s' (after tagging rules); ignoring it.\n",logerror_relation(id),v);
 
           recognised=1; break;
          }
@@ -1143,7 +919,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
           if(ISTRUE(v))
              routes|=Transports_Foot;
           else if(!ISFALSE(v))
-             logerror("Relation %"Prelation_t" has an unrecognised tag 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+             logerror("Relation %"Prelation_t" has an unrecognised tag 'footroute' = '%s' (after tagging rules); using 'no'.\n",logerror_relation(id),v);
           recognised=1; break;
          }
 
@@ -1161,7 +937,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
           if(!strcmp(v,"only_straight_on")) restriction=TurnRestrict_only_straight_on;
 
           if(restriction==TurnRestrict_None)
-             logerror("Relation %"Prelation_t" has an unrecognised tag 'restriction' = '%s' (after tagging rules); ignoring it.\n",id,v);
+             logerror("Relation %"Prelation_t" has an unrecognised tag 'restriction' = '%s' (after tagging rules); ignoring it.\n",logerror_relation(id),v);
 
           recognised=1; break;
          }
@@ -1185,7 +961,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
       }
 
     if(!recognised)
-       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_relation(id),k,v);
    }
 
  /* Create the route relation (must store all relations that have ways or
@@ -1194,6 +970,7 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
 
  if((relation_nways || relation_nrelations) && !relation_turn_restriction)
     AppendRouteRelationList(relations,id,routes,
+                            relation_nodes,relation_nnodes,
                             relation_ways,relation_nways,
                             relation_relations,relation_nrelations);
 
@@ -1202,12 +979,28 @@ void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
  if(relation_turn_restriction && restriction!=TurnRestrict_None)
    {
     if(relation_from==NO_WAY_ID)
-       logerror("Relation %"Prelation_t" is a turn restriction but has no 'from' way.\n",id);
-    else if(relation_to==NO_WAY_ID)
-       logerror("Relation %"Prelation_t" is a turn restriction but has no 'to' way.\n",id);
-    else if(relation_via==NO_NODE_ID)
-       logerror("Relation %"Prelation_t" is a turn restriction but has no 'via' node.\n",id);
-    else
+      {
+       /* Extra logerror information since relation isn't stored */
+       if(relation_to!=NO_WAY_ID) logerror_way(relation_to);
+       if(relation_via!=NO_NODE_ID) logerror_node(relation_via);
+       logerror("Relation %"Prelation_t" is a turn restriction but has no 'from' way.\n",logerror_relation(id));
+      }
+    if(relation_to==NO_WAY_ID)
+      {
+       /* Extra logerror information since relation isn't stored */
+       if(relation_via!=NO_NODE_ID) logerror_node(relation_via);
+       if(relation_from!=NO_WAY_ID) logerror_way(relation_from);
+       logerror("Relation %"Prelation_t" is a turn restriction but has no 'to' way.\n",logerror_relation(id));
+      }
+    if(relation_via==NO_NODE_ID)
+      {
+       /* Extra logerror information since relation isn't stored */
+       if(relation_to!=NO_WAY_ID) logerror_way(relation_to);
+       if(relation_from!=NO_WAY_ID) logerror_way(relation_from);
+       logerror("Relation %"Prelation_t" is a turn restriction but has no 'via' node.\n",logerror_relation(id));
+      }
+
+    if(relation_from!=NO_WAY_ID && relation_to!=NO_WAY_ID && relation_via!=NO_NODE_ID)
        AppendTurnRelationList(relations,id,
                               relation_from,relation_to,relation_via,
                               restriction,except);
@@ -1233,7 +1026,7 @@ static double parse_speed(way_t id,const char *k,const char *v)
  double value=strtod(v,&ev);
 
  if(v==ev)
-    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
  else
    {
     while(isspace(*ev)) ev++;
@@ -1244,7 +1037,7 @@ static double parse_speed(way_t id,const char *k,const char *v)
     if(*ev==0 || !strcmp(ev,"kph"))
        return(value);
 
-    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
    }
 
  return(0);
@@ -1269,7 +1062,7 @@ static double parse_weight(way_t id,const char *k,const char *v)
  double value=strtod(v,&ev);
 
  if(v==ev)
-    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
  else
    {
     while(isspace(*ev)) ev++;
@@ -1282,7 +1075,7 @@ static double parse_weight(way_t id,const char *k,const char *v)
               || !strcmp(ev,"tonne") || !strcmp(ev,"tonnes"))
        return(value);
 
-    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
    }
 
  return(0);
@@ -1307,7 +1100,7 @@ static double parse_length(way_t id,const char *k,const char *v)
  double value=strtod(v,&ev);
 
  if(v==ev)
-    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
  else
    {
     int en=0;
@@ -1342,7 +1135,7 @@ static double parse_length(way_t id,const char *k,const char *v)
     if(*ev==0 || !strcmp(ev,"m") || !strcmp(ev,"metre") || !strcmp(ev,"metres"))
        return(value);
 
-    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+    logerror("Way %"Pway_t" has an un-parseable tag '%s' = '%s' (after tagging rules); ignoring it.\n",logerror_way(id),k,v);
    }
 
  return(0);
