@@ -25,7 +25,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/time.h>
 
 #include "types.h"
 #include "ways.h"
@@ -40,6 +39,7 @@
 
 #include "files.h"
 #include "logging.h"
+#include "errorlogx.h"
 #include "functions.h"
 #include "osmparser.h"
 #include "tagging.h"
@@ -69,11 +69,10 @@ static void print_usage(int detail,const char *argerr,const char *err);
 
 int main(int argc,char** argv)
 {
- struct timeval start_time;
- NodesX     *Nodes;
- SegmentsX  *Segments,*SuperSegments=NULL,*MergedSegments=NULL;
- WaysX      *Ways;
- RelationsX *Relations;
+ NodesX     *OSMNodes;
+ SegmentsX  *OSMSegments,*SuperSegments=NULL,*MergedSegments=NULL;
+ WaysX      *OSMWays;
+ RelationsX *OSMRelations;
  int         iteration=0,quit=0;
  int         max_iterations=5;
  char       *dirname=NULL,*prefix=NULL,*tagging=NULL,*errorlog=NULL;
@@ -83,7 +82,7 @@ int main(int argc,char** argv)
  int         option_prune_isolated=500,option_prune_short=5,option_prune_straight=3;
  int         arg;
 
- gettimeofday(&start_time,NULL);
+ printf_program_start();
 
  /* Parse the command line arguments */
 
@@ -184,7 +183,7 @@ int main(int argc,char** argv)
        if(!ExistsFile(tagging))
          {
           fprintf(stderr,"Error: The '--tagging' option specifies a file that does not exist.\n");
-          return(1);
+          exit(EXIT_FAILURE);
          }
       }
     else
@@ -196,31 +195,29 @@ int main(int argc,char** argv)
        else
          {
           fprintf(stderr,"Error: The '--tagging' option was not used and the default 'tagging.xml' does not exist.\n");
-          return(1);
+          exit(EXIT_FAILURE);
          }
       }
 
     if(ParseXMLTaggingRules(tagging))
       {
        fprintf(stderr,"Error: Cannot read the tagging rules in the file '%s'.\n",tagging);
-       return(1);
+       exit(EXIT_FAILURE);
       }
    }
 
  /* Create new node, segment, way and relation variables */
 
- Nodes=NewNodeList(option_append||option_changes,option_process_only);
+ OSMNodes=NewNodeList(option_append||option_changes,option_process_only);
 
- Segments=NewSegmentList(option_append||option_changes,option_process_only);
+ OSMWays=NewWayList(option_append||option_changes,option_process_only);
 
- Ways=NewWayList(option_append||option_changes,option_process_only);
-
- Relations=NewRelationList(option_append||option_changes,option_process_only);
+ OSMRelations=NewRelationList(option_append||option_changes,option_process_only);
 
  /* Create the error log file */
 
  if(errorlog)
-    open_errorlog(FileName(dirname,prefix,errorlog),option_append||option_changes||option_process_only);
+    open_errorlog(FileName(dirname,prefix,errorlog),option_append||option_changes||option_process_only,option_keep);
 
  /* Parse the file */
 
@@ -236,7 +233,7 @@ if(!option_process_only)
 
       filename=strcpy(malloc(strlen(argv[arg])+1),argv[arg]);
 
-      fd=ReOpenFile(filename);
+      fd=OpenFile(filename);
 
       if((p=strstr(filename,".bz2")) && !strcmp(p,".bz2"))
         {
@@ -261,12 +258,12 @@ if(!option_process_only)
            }
          else if((p=strstr(filename,".o5c")) && !strcmp(p,".o5c"))
            {
-            if(ParseO5CFile(fd,Nodes,Segments,Ways,Relations))
+            if(ParseO5CFile(fd,OSMNodes,OSMWays,OSMRelations))
                exit(EXIT_FAILURE);
            }
          else
            {
-            if(ParseOSCFile(fd,Nodes,Segments,Ways,Relations))
+            if(ParseOSCFile(fd,OSMNodes,OSMWays,OSMRelations))
                exit(EXIT_FAILURE);
            }
         }
@@ -277,17 +274,17 @@ if(!option_process_only)
 
          if((p=strstr(filename,".pbf")) && !strcmp(p,".pbf"))
            {
-            if(ParsePBFFile(fd,Nodes,Segments,Ways,Relations))
+            if(ParsePBFFile(fd,OSMNodes,OSMWays,OSMRelations))
                exit(EXIT_FAILURE);
            }
          else if((p=strstr(filename,".o5m")) && !strcmp(p,".o5m"))
            {
-            if(ParseO5MFile(fd,Nodes,Segments,Ways,Relations))
+            if(ParseO5MFile(fd,OSMNodes,OSMWays,OSMRelations))
                exit(EXIT_FAILURE);
            }
          else
            {
-            if(ParseOSMFile(fd,Nodes,Segments,Ways,Relations))
+            if(ParseOSMFile(fd,OSMNodes,OSMWays,OSMRelations))
                exit(EXIT_FAILURE);
            }
         }
@@ -300,17 +297,15 @@ if(!option_process_only)
    DeleteXMLTaggingRules();
   }
 
- FinishNodeList(Nodes);
- FinishSegmentList(Segments);
- FinishWayList(Ways);
- FinishRelationList(Relations);
+ FinishNodeList(OSMNodes);
+ FinishWayList(OSMWays);
+ FinishRelationList(OSMRelations);
 
  if(option_parse_only)
    {
-    FreeNodeList(Nodes,1);
-    FreeSegmentList(Segments,1);
-    FreeWayList(Ways,1);
-    FreeRelationList(Relations,1);
+    FreeNodeList(OSMNodes,1);
+    FreeWayList(OSMWays,1);
+    FreeRelationList(OSMRelations,1);
 
     return(0);
    }
@@ -321,61 +316,50 @@ if(!option_process_only)
  printf("\nSort OSM Data\n=============\n\n");
  fflush(stdout);
 
- /* Sort the nodes, segments, ways and relations */
+ /* Sort the nodes, ways and relations */
 
- SortNodeList(Nodes);
+ SortNodeList(OSMNodes);
 
- if(option_changes)
-    ApplySegmentChanges(Segments);
+ SortWayList(OSMWays);
 
- SortSegmentList(Segments);
-
- SortWayList(Ways);
-
- SortRelationList(Relations);
+ SortRelationList(OSMRelations);
 
  /* Process the data */
 
  printf("\nProcess OSM Data\n================\n\n");
  fflush(stdout);
 
- /* Extract the way names (must be before using the ways) */
+ /* Remove non-highway nodes by looking through the ways */
 
- ExtractWayNames(Ways,option_keep||option_changes);
+ RemoveNonHighwayNodes(OSMNodes,OSMWays,option_keep||option_changes);
 
- /* Remove bad segments (must be after sorting the nodes, segments and ways) */
+ /* Separate the segments and way names and sort them. */
 
- RemoveBadSegments(Segments,Nodes,Ways,option_keep||option_changes);
+ OSMSegments=SplitWays(OSMWays,OSMNodes,option_keep||option_changes);
 
- /* Remove non-highway nodes (must be after removing the bad segments) */
+ SortWayNames(OSMWays);
 
- RemoveNonHighwayNodes(Nodes,Segments,option_keep||option_changes);
+ SortSegmentList(OSMSegments);
 
- /* Process the route relations and first part of turn relations (must be before compacting the ways) */
+ /* Process the segments and index them */
 
- ProcessRouteRelations(Relations,Ways,option_keep||option_changes);
+ ProcessSegments(OSMSegments,OSMNodes,OSMWays);
 
- ProcessTurnRelations1(Relations,Nodes,Ways,option_keep||option_changes);
+ IndexSegments(OSMSegments,OSMNodes,OSMWays);
 
- /* Measure the segments and replace node/way id with index (must be after removing non-highway nodes) */
+ /* Process the route relations and turn relations (must be before compacting the ways) */
 
- MeasureSegments(Segments,Nodes,Ways);
+ ProcessRouteRelations(OSMRelations,OSMWays,option_keep||option_changes);
+
+ ProcessTurnRelations(OSMRelations,OSMNodes,OSMSegments,OSMWays,option_keep||option_changes);
+
+ /* Compact the ways (must be after processing turn relations) */
+
+ CompactWayList(OSMWays,OSMSegments);
 
  /* Index the segments */
 
- IndexSegments(Segments,Nodes,Ways);
-
- /* Convert the turn relations from ways into nodes */
-
- ProcessTurnRelations2(Relations,Nodes,Segments,Ways);
-
- /* Compact the ways (must be after turn relations 2) */
-
- CompactWayList(Ways,Segments);
-
- /* Index the segments */
-
- IndexSegments(Segments,Nodes,Ways);
+ IndexSegments(OSMSegments,OSMNodes,OSMWays);
 
  /* Prune unwanted nodes/segments. */
 
@@ -384,33 +368,34 @@ if(!option_process_only)
     printf("\nPrune Unneeded Data\n===================\n\n");
     fflush(stdout);
 
-    StartPruning(Nodes,Segments,Ways);
-
-    if(option_prune_isolated)
-       PruneIsolatedRegions(Nodes,Segments,Ways,option_prune_isolated);
-
-    if(option_prune_short)
-       PruneShortSegments(Nodes,Segments,Ways,option_prune_short);
+    StartPruning(OSMNodes,OSMSegments,OSMWays);
 
     if(option_prune_straight)
-       PruneStraightHighwayNodes(Nodes,Segments,Ways,option_prune_straight);
+       PruneStraightHighwayNodes(OSMNodes,OSMSegments,OSMWays,option_prune_straight);
 
-    FinishPruning(Nodes,Segments,Ways);
+    if(option_prune_isolated)
+       PruneIsolatedRegions(OSMNodes,OSMSegments,OSMWays,option_prune_isolated);
 
-    /* Remove the pruned nodes and segments and update the indexes */
+    if(option_prune_short)
+       PruneShortSegments(OSMNodes,OSMSegments,OSMWays,option_prune_short);
 
-    RemovePrunedNodes(Nodes,Segments);
-    RemovePrunedSegments(Segments,Ways);
-    CompactWayList(Ways,Segments);
-    RemovePrunedTurnRelations(Relations,Nodes);
-    IndexSegments(Segments,Nodes,Ways);
+    FinishPruning(OSMNodes,OSMSegments,OSMWays);
+
+    /* Remove the pruned nodes, segments, ways and relations and update the indexes */
+
+    RemovePrunedNodes(OSMNodes,OSMSegments);
+    RemovePrunedSegments(OSMSegments,OSMWays);
+    CompactWayList(OSMWays,OSMSegments);
+    RemovePrunedTurnRelations(OSMRelations,OSMNodes);
+
+    IndexSegments(OSMSegments,OSMNodes,OSMWays);
    }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
 
  do
    {
-    int nsuper;
+    index_t nsuper;
 
     printf("\nProcess Super-Data (iteration %d)\n================================%s\n\n",iteration,iteration>9?"=":"");
     fflush(stdout);
@@ -419,13 +404,13 @@ if(!option_process_only)
       {
        /* Select the super-nodes */
 
-       ChooseSuperNodes(Nodes,Segments,Ways);
+       ChooseSuperNodes(OSMNodes,OSMSegments,OSMWays);
 
        /* Select the super-segments */
 
-       SuperSegments=CreateSuperSegments(Nodes,Segments,Ways);
+       SuperSegments=CreateSuperSegments(OSMNodes,OSMSegments,OSMWays);
 
-       nsuper=Segments->number;
+       nsuper=OSMSegments->number;
       }
     else
       {
@@ -433,26 +418,26 @@ if(!option_process_only)
 
        /* Select the super-nodes */
 
-       ChooseSuperNodes(Nodes,SuperSegments,Ways);
+       ChooseSuperNodes(OSMNodes,SuperSegments,OSMWays);
 
        /* Select the super-segments */
 
-       SuperSegments2=CreateSuperSegments(Nodes,SuperSegments,Ways);
+       SuperSegments2=CreateSuperSegments(OSMNodes,SuperSegments,OSMWays);
 
        nsuper=SuperSegments->number;
 
-       FreeSegmentList(SuperSegments,0);
+       FreeSegmentList(SuperSegments);
 
        SuperSegments=SuperSegments2;
       }
 
     /* Sort the super-segments and remove duplicates */
 
-    DeduplicateSuperSegments(SuperSegments,Ways);
+    DeduplicateSuperSegments(SuperSegments,OSMWays);
 
     /* Index the segments */
 
-    IndexSegments(SuperSegments,Nodes,Ways);
+    IndexSegments(SuperSegments,OSMNodes,OSMWays);
 
     /* Check for end condition */
 
@@ -473,17 +458,13 @@ if(!option_process_only)
 
  /* Merge the super-segments */
 
- MergedSegments=MergeSuperSegments(Segments,SuperSegments);
+ MergedSegments=MergeSuperSegments(OSMSegments,SuperSegments);
 
- FreeSegmentList(Segments,0);
+ FreeSegmentList(OSMSegments);
 
- FreeSegmentList(SuperSegments,0);
+ FreeSegmentList(SuperSegments);
 
- Segments=MergedSegments;
-
- /* Re-index the merged segments */
-
- IndexSegments(Segments,Nodes,Ways);
+ OSMSegments=MergedSegments;
 
  /* Cross reference the nodes and segments */
 
@@ -492,17 +473,17 @@ if(!option_process_only)
 
  /* Sort the nodes and segments geographically */
 
- SortNodeListGeographically(Nodes);
+ SortNodeListGeographically(OSMNodes);
 
- SortSegmentListGeographically(Segments,Nodes);
+ SortSegmentListGeographically(OSMSegments,OSMNodes);
 
  /* Re-index the segments */
 
- IndexSegments(Segments,Nodes,Ways);
+ IndexSegments(OSMSegments,OSMNodes,OSMWays);
 
  /* Sort the turn relations geographically */
 
- SortTurnRelationListGeographically(Relations,Nodes,Segments);
+ SortTurnRelationListGeographically(OSMRelations,OSMNodes,OSMSegments);
 
  /* Output the results */
 
@@ -511,43 +492,55 @@ if(!option_process_only)
 
  /* Write out the nodes */
 
- SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"),Segments);
-
- FreeNodeList(Nodes,0);
+ SaveNodeList(OSMNodes,FileName(dirname,prefix,"nodes.mem"),OSMSegments);
 
  /* Write out the segments */
 
- SaveSegmentList(Segments,FileName(dirname,prefix,"segments.mem"));
-
- FreeSegmentList(Segments,0);
+ SaveSegmentList(OSMSegments,FileName(dirname,prefix,"segments.mem"));
 
  /* Write out the ways */
 
- SaveWayList(Ways,FileName(dirname,prefix,"ways.mem"));
-
- FreeWayList(Ways,0);
+ SaveWayList(OSMWays,FileName(dirname,prefix,"ways.mem"));
 
  /* Write out the relations */
 
- SaveRelationList(Relations,FileName(dirname,prefix,"relations.mem"));
+ SaveRelationList(OSMRelations,FileName(dirname,prefix,"relations.mem"));
 
- FreeRelationList(Relations,0);
-
- /* Close the error log file */
+ /* Close the error log file and process the data */
 
  if(errorlog)
+   {
     close_errorlog();
 
- /* Print the total time */
+    if(option_keep)
+      {
+       ErrorLogsX *OSMErrorLogs;
 
- if(option_logtime)
-   {
-    printf("\n");
-    fprintf_elapsed_time(stdout,&start_time);
-    printf("Complete\n");
-    fflush(stdout);
+       printf("\nCreate Error Log\n================\n\n");
+       fflush(stdout);
+
+       OSMErrorLogs=NewErrorLogList();
+
+       ProcessErrorLogs(OSMErrorLogs,OSMNodes,OSMWays,OSMRelations);
+
+       SortErrorLogsGeographically(OSMErrorLogs);
+
+       SaveErrorLogs(OSMErrorLogs,FileName(dirname,prefix,"errorlogs.mem"));
+
+       FreeErrorLogList(OSMErrorLogs);
+      }
    }
 
+ /* Free the memory (delete the temporary files) */
+
+ FreeNodeList(OSMNodes,0);
+ FreeWayList(OSMWays,0);
+ FreeRelationList(OSMRelations,0);
+
+ FreeSegmentList(OSMSegments);
+
+ printf_program_end();
+
  return(0);
 }
 
@@ -585,7 +578,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "                      [--prune-straight=<len>]\n"
          "                      [<filename.osm> ... | <filename.osc> ...\n"
          "                       | <filename.pbf> ...\n"
-         "                       | <filename.osm> ... | <filename.osc> ..."
+         "                       | <filename.o5m> ... | <filename.o5c> ..."
 #if defined(USE_BZIP2) && USE_BZIP2
          "\n                       | <filename.(osm|osc|o5m|o5c).bz2> ..."
 #endif
