@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -20,14 +20,15 @@
  ***************************************/
 
 
-#include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "results.h"
 
-/*+ The size of the increment for the Results data structure. +*/
-#define RESULTS_INCREMENT 64
+#include "logging.h"
+
+
+#define HASH_NODE_SEGMENT(node,segment) ((node)^(segment<<4))
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -35,42 +36,29 @@
 
   Results *NewResultsList Returns the results list.
 
-  int nbins The number of bins in the results array.
+  uint8_t log2bins The base 2 logarithm of the initial number of bins in the results array.
   ++++++++++++++++++++++++++++++++++++++*/
 
-Results *NewResultsList(int nbins)
+Results *NewResultsList(uint8_t log2bins)
 {
  Results *results;
- uint32_t i;
 
  results=(Results*)malloc(sizeof(Results));
 
- results->nbins=1;
- results->mask=~0;
+ results->nbins=1<<log2bins;
+ results->mask=results->nbins-1;
+ results->ncollisions=log2bins-4;
 
- while(nbins>>=1)
-   {
-    results->mask<<=1;
-    results->nbins<<=1;
-   }
-
- results->mask=~results->mask;
-
- results->alloced=RESULTS_INCREMENT;
  results->number=0;
 
- results->count=(uint32_t*)malloc(results->nbins*sizeof(uint32_t));
- results->point=(Result***)malloc(results->nbins*sizeof(Result**));
+ results->count=(uint8_t*)calloc(results->nbins,sizeof(uint8_t));
+ results->point=(Result**)calloc(results->nbins,sizeof(Result*));
 
- for(i=0;i<results->nbins;i++)
-   {
-    results->count[i]=0;
+ results->ndata1=0;
+ results->nallocdata1=0;
+ results->ndata2=results->nbins>>2;
 
-    results->point[i]=(Result**)malloc(results->alloced*sizeof(Result*));
-   }
-
- results->data=(Result**)malloc(1*sizeof(Result*));
- results->data[0]=(Result*)malloc(results->nbins*RESULTS_INCREMENT*sizeof(Result));
+ results->data=NULL;
 
  results->start_node=NO_NODE;
  results->prev_segment=NO_SEGMENT;
@@ -82,6 +70,33 @@ Results *NewResultsList(int nbins)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Allocate a new results list.
+
+  Results *results The results list to be reset.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void ResetResultsList(Results *results)
+{
+ uint32_t i;
+
+ results->number=0;
+ results->ndata1=0;
+
+ for(i=0;i<results->nbins;i++)
+   {
+    results->point[i]=NULL;
+    results->count[i]=0;
+   }
+
+ results->start_node=NO_NODE;
+ results->prev_segment=NO_SEGMENT;
+
+ results->finish_node=NO_NODE;
+ results->last_segment=NO_SEGMENT;
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Free a results list.
 
@@ -90,16 +105,13 @@ Results *NewResultsList(int nbins)
 
 void FreeResultsList(Results *results)
 {
- int i,c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
+ uint32_t i;
 
- for(i=c;i>=0;i--)
+ for(i=0;i<results->nallocdata1;i++)
     free(results->data[i]);
 
  free(results->data);
 
- for(i=0;i<results->nbins;i++)
-    free(results->point[i]);
-
  free(results->point);
 
  free(results->count);
@@ -123,38 +135,83 @@ void FreeResultsList(Results *results)
 Result *InsertResult(Results *results,index_t node,index_t segment)
 {
  Result *result;
- int bin=node&results->mask;
- uint32_t i;
+ uint32_t bin=HASH_NODE_SEGMENT(node,segment)&results->mask;
 
- /* Check that the arrays have enough space or allocate more. */
+ /* Check if we have hit the limit on the number of collisions per bin */
 
- if(results->count[bin]==results->alloced)
+ if(results->count[bin]==results->ncollisions)
    {
-    results->alloced+=RESULTS_INCREMENT;
+    uint32_t i;
 
-    for(i=0;i<results->nbins;i++)
-       results->point[i]=(Result**)realloc((void*)results->point[i],results->alloced*sizeof(Result*));
+    results->nbins<<=1;
+    results->mask=results->nbins-1;
+    results->ncollisions++;
+
+    results->count=(uint8_t*)realloc((void*)results->count,results->nbins*sizeof(uint8_t));
+    results->point=(Result**)realloc((void*)results->point,results->nbins*sizeof(Result*));
+
+    for(i=0;i<results->nbins/2;i++)
+      {
+       Result *r=results->point[i];
+       Result **bin1,**bin2;
+
+       results->count[i]                 =0;
+       results->count[i+results->nbins/2]=0;
+
+       bin1=&results->point[i];
+       bin2=&results->point[i+results->nbins/2];
+
+       *bin1=NULL;
+       *bin2=NULL;
+
+       while(r)
+         {
+          Result *rh=r->hashnext;
+          uint32_t newbin=HASH_NODE_SEGMENT(r->node,r->segment)&results->mask;
+
+          r->hashnext=NULL;
+
+          if(newbin==i)
+            { *bin1=r; bin1=&r->hashnext; }
+          else
+            { *bin2=r; bin2=&r->hashnext; }
+
+          results->count[newbin]++;
+
+          r=rh;
+         }
+      }
+
+    bin=HASH_NODE_SEGMENT(node,segment)&results->mask;
    }
 
- if(results->number && (results->number%RESULTS_INCREMENT)==0 && (results->number%(RESULTS_INCREMENT*results->nbins))==0)
-   {
-    int c=results->number/(results->nbins*RESULTS_INCREMENT);
+ /* Check if we need more data space allocated */
 
-    results->data=(Result**)realloc((void*)results->data,(c+1)*sizeof(Result*));
-    results->data[c]=(Result*)malloc(results->nbins*RESULTS_INCREMENT*sizeof(Result));
+ if((results->number%results->ndata2)==0)
+   {
+    results->ndata1++;
+
+    if(results->ndata1>=results->nallocdata1)
+      {
+       results->nallocdata1++;
+       results->data=(Result**)realloc((void*)results->data,results->nallocdata1*sizeof(Result*));
+       results->data[results->nallocdata1-1]=(Result*)malloc(results->ndata2*sizeof(Result));
+      }
    }
 
  /* Insert the new entry */
 
- results->point[bin][results->count[bin]]=&results->data[results->number/(results->nbins*RESULTS_INCREMENT)][results->number%(results->nbins*RESULTS_INCREMENT)];
+ result=&results->data[results->ndata1-1][results->number%results->ndata2];
 
- results->number++;
+ result->hashnext=results->point[bin];
+
+ results->point[bin]=result;
 
  results->count[bin]++;
 
- /* Initialise the result */
+ results->number++;
 
- result=results->point[bin][results->count[bin]-1];
+ /* Initialise the result */
 
  result->node=node;
  result->segment=segment;
@@ -171,34 +228,6 @@ Result *InsertResult(Results *results,index_t node,index_t segment)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Find a result; search by node only (don't care about the segment but find the shortest).
-
-  Result *FindResult1 Returns the result that has been found.
-
-  Results *results The results structure to search.
-
-  index_t node The node that is to be found.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-Result *FindResult1(Results *results,index_t node)
-{
- int bin=node&results->mask;
- score_t best_score=INF_SCORE;
- Result *best_result=NULL;
- int i;
-
- for(i=results->count[bin]-1;i>=0;i--)
-    if(results->point[bin][i]->node==node && results->point[bin][i]->score<best_score)
-      {
-       best_score=results->point[bin][i]->score;
-       best_result=results->point[bin][i];
-      }
-
- return(best_result);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Find a result; search by node and segment.
 
@@ -213,14 +242,20 @@ Result *FindResult1(Results *results,index_t node)
 
 Result *FindResult(Results *results,index_t node,index_t segment)
 {
- int bin=node&results->mask;
- int i;
+ Result *r;
+ uint32_t bin=HASH_NODE_SEGMENT(node,segment)&results->mask;
 
- for(i=results->count[bin]-1;i>=0;i--)
-    if(results->point[bin][i]->node==node && results->point[bin][i]->segment==segment)
-       return(results->point[bin][i]);
+ r=results->point[bin];
 
- return(NULL);
+ while(r)
+   {
+    if(r->segment==segment && r->node==node)
+       break;
+
+    r=r->hashnext;
+   }
+
+ return(r);
 }
 
 
@@ -250,20 +285,22 @@ Result *FirstResult(Results *results)
 
 Result *NextResult(Results *results,Result *result)
 {
- int i,j=0,c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
+ uint32_t i;
+ size_t j=0;
 
- for(i=0;i<=c;i++)
-   {
-    j=result-results->data[i];
+ for(i=0;i<results->ndata1;i++)
+    if(result>=results->data[i])
+      {
+       j=result-results->data[i];
 
-    if(j>=0 && j<(results->nbins*RESULTS_INCREMENT))
-       break;
-   }
+       if(j<results->ndata2)
+          break;
+      }
 
- if(++j>=(results->nbins*RESULTS_INCREMENT))
+ if(++j>=results->ndata2)
    {i++;j=0;}
 
- if((i*(results->nbins*RESULTS_INCREMENT)+j)>=results->number)
+ if((i*results->ndata2+j)>=results->number)
     return(NULL);
 
  return(&results->data[i][j]);
