@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -31,7 +31,7 @@
 /* Constants */
 
 /*+ A result is not currently queued. +*/
-#define NOT_QUEUED (uint32_t)(~0)
+#define NOT_QUEUED (uint32_t)(0)
 
 
 /* Data structures */
@@ -51,25 +51,30 @@ struct _Result
  score_t   sortby;              /*+ The best possible weighted distance or duration score from the start to the finish. +*/
 
  uint32_t  queued;              /*+ The position of this result in the queue. +*/
+
+ Result   *hashnext;            /*+ The next result in the linked list for this hash bin. +*/
 };
 
 /*+ A list of results. +*/
 typedef struct _Results
 {
- uint32_t  nbins;               /*+ The number of bins. +*/
- uint32_t  mask;                /*+ A bit mask to select the bottom 'nbins' bits. +*/
+ uint32_t  nbins;               /*+ The number of bins in the has table. +*/
+ uint32_t  mask;                /*+ A bit mask to select the bottom log2(nbins) bits. +*/
 
- uint32_t  alloced;             /*+ The amount of space allocated for results
-                                    (the length of the number and pointers arrays and
-                                     1/nbins times the amount in the real results). +*/
  uint32_t  number;              /*+ The total number of occupied results. +*/
 
- uint32_t *count;               /*+ An array of nbins counters of results in each array. +*/
- Result ***point;               /*+ An array of nbins arrays of pointers to actual results. +*/
+ uint8_t   ncollisions;         /*+ The number of results allowed in each hash bin. +*/
+ uint8_t  *count;               /*+ An array of nbins counters of results in each hash bin. +*/
 
- Result  **data;                /*+ An array of arrays containing the actual results
-                                    (don't need to realloc the array of data when adding more,
-                                    only realloc the array that points to the array of data).
+ Result  **point;               /*+ An array of nbins linked lists of results for one hash bin. +*/
+
+ uint32_t  ndata1;              /*+ The size of the first dimension of the 'data' array. +*/
+ uint32_t  ndata2;              /*+ The size of the second dimension of the 'data' array. +*/
+
+ uint32_t  nallocdata1;         /*+ The amount of allocated space in the first dimension of the 'data' array. +*/
+
+ Result  **data;                /*+ An array of arrays containing the actual results, the first
+                                    dimension is reallocated but the second dimension is not.
                                     Most importantly pointers into the real data don't change
                                     as more space is allocated (since realloc is not being used). +*/
 
@@ -89,12 +94,12 @@ typedef struct _Queue Queue;
 
 /* Results functions in results.c */
 
-Results *NewResultsList(int nbins);
+Results *NewResultsList(uint8_t log2bins);
+void ResetResultsList(Results *results);
 void FreeResultsList(Results *results);
 
 Result *InsertResult(Results *results,index_t node,index_t segment);
 
-Result *FindResult1(Results *results,index_t node);
 Result *FindResult(Results *results,index_t node,index_t segment);
 
 Result *FirstResult(Results *results);
@@ -103,10 +108,11 @@ Result *NextResult(Results *results,Result *result);
 
 /* Queue functions in queue.c */
 
-Queue *NewQueueList(void);
+Queue *NewQueueList(uint8_t log2bins);
+void ResetQueueList(Queue *queue);
 void FreeQueueList(Queue *queue);
 
-void InsertInQueue(Queue *queue,Result *result);
+void InsertInQueue(Queue *queue,Result *result,score_t score);
 Result *PopFromQueue(Queue *queue);
 
 
