@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdlib.h>
 
 #include "types.h"
@@ -40,7 +39,7 @@
 
 /* Local functions */
 
-static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,node_t start,Way *match);
+static Results *FindSuperRoutes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,node_t start,Way *match);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -71,35 +70,39 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
    {
     nodesx->super=AllocBitMask(nodesx->number);
 
-    assert(nodesx->super); /* Check AllocBitMask() worked */
+    logassert(nodesx->super,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
-    SetAllBits1(nodesx->super,nodesx->number);
+    SetAllBits(nodesx->super,nodesx->number);
    }
 
  /* Map into memory / open the files */
 
+ nodesx->fd=ReOpenFileBuffered(nodesx->filename_tmp);
+
 #if !SLIM
- nodesx->data=MapFile(nodesx->filename);
- segmentsx->data=MapFile(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ segmentsx->data=MapFile(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename);
- segmentsx->fd=ReOpenFile(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ segmentsx->fd=SlimMapFile(segmentsx->filename_tmp);
+ waysx->fd=SlimMapFile(waysx->filename_tmp);
+
+ InvalidateSegmentXCache(segmentsx->cache);
+ InvalidateWayXCache(waysx->cache);
 #endif
 
  /* Find super-nodes */
 
  for(i=0;i<nodesx->number;i++)
    {
+    NodeX nodex;
+
+    ReadFileBuffered(nodesx->fd,&nodex,sizeof(NodeX));
+
     if(IsBitSet(nodesx->super,i))
       {
        int issuper=0;
-       NodeX *nodex=LookupNodeX(nodesx,i,1);
 
-       if(IsPrunedNodeX(nodex))
-          issuper=0;
-       else if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
+       if(nodex.flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
           issuper=1;
        else
          {
@@ -115,7 +118,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
              /* Segments that are loops count twice */
 
-             assert(count<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
+             logassert(count<MAX_SEG_PER_NODE,"Too many segments for one node (increase MAX_SEG_PER_NODE?)"); /* Only a limited amount of information stored. */
 
              if(segmentx->node1==segmentx->node2)
                 segmentweight[count]=2;
@@ -126,7 +129,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
              /* If the node allows less traffic types than any connecting way then it is super if it allows anything */
 
-             if((wayx->way.allow&nodex->allow)!=wayx->way.allow && nodex->allow!=Transports_None)
+             if((wayx->way.allow&nodex.allow)!=wayx->way.allow && nodex.allow!=Transports_None)
                {
                 issuper=1;
                 break;
@@ -179,15 +182,15 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  /* Unmap from memory / close the files */
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
- nodesx->fd=CloseFile(nodesx->fd);
- segmentsx->fd=CloseFile(segmentsx->fd);
- waysx->fd=CloseFile(waysx->fd);
+ segmentsx->fd=SlimUnmapFile(segmentsx->fd);
+ waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
+ nodesx->fd=CloseFileBuffered(nodesx->fd);
+
  /* Print the final message */
 
  printf_last("Found Super-Nodes: Nodes=%"Pindex_t" Super-Nodes=%"Pindex_t,nodesx->number,nnodes);
@@ -212,10 +215,14 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  SegmentsX *supersegmentsx;
  index_t sn=0,ss=0;
 
- supersegmentsx=NewSegmentList(0);
+ supersegmentsx=NewSegmentList();
 
  if(segmentsx->number==0 || waysx->number==0)
+   {
+    FinishSegmentList(supersegmentsx);
+
     return(supersegmentsx);
+   }
 
  /* Print the start message */
 
@@ -224,13 +231,17 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  /* Map into memory / open the files */
 
 #if !SLIM
- nodesx->data=MapFile(nodesx->filename);
- segmentsx->data=MapFile(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ nodesx->data=MapFile(nodesx->filename_tmp);
+ segmentsx->data=MapFile(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename);
- segmentsx->fd=ReOpenFile(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ nodesx->fd=SlimMapFile(nodesx->filename_tmp);
+ segmentsx->fd=SlimMapFile(segmentsx->filename_tmp);
+ waysx->fd=SlimMapFile(waysx->filename_tmp);
+
+ InvalidateNodeXCache(nodesx->cache);
+ InvalidateSegmentXCache(segmentsx->cache);
+ InvalidateWayXCache(waysx->cache);
 #endif
 
  /* Create super-segments for each super-node. */
@@ -265,7 +276,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
                   }
             }
 
-          assert(count<MAX_SEG_PER_NODE); /* Only a limited amount of history stored. */
+          logassert(count<MAX_SEG_PER_NODE,"Too many segments for one node (increase MAX_SEG_PER_NODE?)"); /* Only a limited amount of history stored. */
 
           prevway[count++]=wayx->way;
 
@@ -273,25 +284,23 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
           if(!match)
             {
-             Results *results=FindRoutesWay(nodesx,segmentsx,waysx,i,&wayx->way);
+             Results *results=FindSuperRoutes(nodesx,segmentsx,waysx,i,&wayx->way);
              Result *result=FirstResult(results);
 
              while(result)
                {
                 if(IsBitSet(nodesx->super,result->node) && result->segment!=NO_SEGMENT)
                   {
-                   if(wayx->way.type&Way_OneWay && result->node!=i)
-                      AppendSegment(supersegmentsx,segmentx->way,i,result->node,DISTANCE((distance_t)result->score)|ONEWAY_1TO2);
+                   if(wayx->way.type&Highway_OneWay && result->node!=i)
+                      AppendSegmentList(supersegmentsx,segmentx->way,i,result->node,DISTANCE((distance_t)result->score)|ONEWAY_1TO2);
                    else
-                      AppendSegment(supersegmentsx,segmentx->way,i,result->node,DISTANCE((distance_t)result->score));
+                      AppendSegmentList(supersegmentsx,segmentx->way,i,result->node,DISTANCE((distance_t)result->score));
 
                    ss++;
                   }
 
                 result=NextResult(results,result);
                }
-
-             FreeResultsList(results);
             }
 
           segmentx=NextSegmentX(segmentsx,segmentx,i);
@@ -304,16 +313,18 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
       }
    }
 
+ FinishSegmentList(supersegmentsx);
+
  /* Unmap from memory / close the files */
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
- nodesx->fd=CloseFile(nodesx->fd);
- segmentsx->fd=CloseFile(segmentsx->fd);
- waysx->fd=CloseFile(waysx->fd);
+ nodesx->fd=SlimUnmapFile(nodesx->fd);
+ segmentsx->fd=SlimUnmapFile(segmentsx->fd);
+ waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
  /* Print the final message */
@@ -336,45 +347,54 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
 SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 {
- index_t i,j;
+ index_t i,j,lastj;
  index_t merged=0,added=0;
  SegmentsX *mergedsegmentsx;
 
- mergedsegmentsx=NewSegmentList(0);
+ mergedsegmentsx=NewSegmentList();
 
  if(segmentsx->number==0)
+   {
+    FinishSegmentList(mergedsegmentsx);
+
     return(mergedsegmentsx);
+   }
 
  /* Print the start message */
 
  printf_first("Merging Segments: Segments=0 Super=0 Merged=0 Added=0");
 
- /* Map into memory / open the files */
+ /* Open the files */
 
-#if !SLIM
- segmentsx->data=MapFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFileBuffered(segmentsx->filename_tmp);
  if(supersegmentsx->number>0)
-    supersegmentsx->data=MapFile(supersegmentsx->filename);
-#else
- segmentsx->fd=ReOpenFile(segmentsx->filename);
- if(supersegmentsx->number>0)
-    supersegmentsx->fd=ReOpenFile(supersegmentsx->filename);
-#endif
+    supersegmentsx->fd=ReOpenFileBuffered(supersegmentsx->filename_tmp);
 
  /* Loop through and create a new list of combined segments */
 
- for(i=0,j=0;i<segmentsx->number;i++)
+ lastj=-1;
+ j=0;
+
+ for(i=0;i<segmentsx->number;i++)
    {
     int super=0;
-    SegmentX *segmentx=LookupSegmentX(segmentsx,i,1);
+    SegmentX segmentx;
+
+    ReadFileBuffered(segmentsx->fd,&segmentx,sizeof(SegmentX));
 
     while(j<supersegmentsx->number)
       {
-       SegmentX *supersegmentx=LookupSegmentX(supersegmentsx,j,1);
+       SegmentX supersegmentx;
 
-       if(segmentx->node1   ==supersegmentx->node1 &&
-          segmentx->node2   ==supersegmentx->node2 &&
-          segmentx->distance==supersegmentx->distance)
+       if(j!=lastj)
+         {
+          ReadFileBuffered(supersegmentsx->fd,&supersegmentx,sizeof(SegmentX));
+          lastj=j;
+         }
+
+       if(segmentx.node1   ==supersegmentx.node1 &&
+          segmentx.node2   ==supersegmentx.node2 &&
+          segmentx.distance==supersegmentx.distance)
          {
           merged++;
           j++;
@@ -382,14 +402,14 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
           super=1;
           break;
          }
-       else if((segmentx->node1==supersegmentx->node1 &&
-                segmentx->node2==supersegmentx->node2) ||
-               (segmentx->node1==supersegmentx->node1 &&
-                segmentx->node2>supersegmentx->node2) ||
-               (segmentx->node1>supersegmentx->node1))
+       else if((segmentx.node1==supersegmentx.node1 &&
+                segmentx.node2==supersegmentx.node2) ||
+               (segmentx.node1==supersegmentx.node1 &&
+                segmentx.node2>supersegmentx.node2) ||
+               (segmentx.node1>supersegmentx.node1))
          {
           /* mark as super-segment */
-          AppendSegment(mergedsegmentsx,supersegmentx->way,supersegmentx->node1,supersegmentx->node2,supersegmentx->distance|SEGMENT_SUPER);
+          AppendSegmentList(mergedsegmentsx,supersegmentx.way,supersegmentx.node1,supersegmentx.node2,supersegmentx.distance|SEGMENT_SUPER);
           added++;
           j++;
          }
@@ -401,25 +421,21 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
       }
 
     if(super)
-       AppendSegment(mergedsegmentsx,segmentx->way,segmentx->node1,segmentx->node2,segmentx->distance|SEGMENT_SUPER|SEGMENT_NORMAL);
+       AppendSegmentList(mergedsegmentsx,segmentx.way,segmentx.node1,segmentx.node2,segmentx.distance|SEGMENT_SUPER|SEGMENT_NORMAL);
     else
-       AppendSegment(mergedsegmentsx,segmentx->way,segmentx->node1,segmentx->node2,segmentx->distance|SEGMENT_NORMAL);
+       AppendSegmentList(mergedsegmentsx,segmentx.way,segmentx.node1,segmentx.node2,segmentx.distance|SEGMENT_NORMAL);
 
     if(!((i+1)%10000))
        printf_middle("Merging Segments: Segments=%"Pindex_t" Super=%"Pindex_t" Merged=%"Pindex_t" Added=%"Pindex_t,i+1,j,merged,added);
    }
 
- /* Unmap from memory / close the files */
+ FinishSegmentList(mergedsegmentsx);
 
-#if !SLIM
- segmentsx->data=UnmapFile(segmentsx->filename);
+ /* Close the files */
+
+ segmentsx->fd=CloseFileBuffered(segmentsx->fd);
  if(supersegmentsx->number>0)
-    supersegmentsx->data=UnmapFile(supersegmentsx->filename);
-#else
- segmentsx->fd=CloseFile(segmentsx->fd);
- if(supersegmentsx->number>0)
-    supersegmentsx->fd=CloseFile(supersegmentsx->fd);
-#endif
+    supersegmentsx->fd=CloseFileBuffered(supersegmentsx->fd);
 
  /* Print the final message */
 
@@ -432,7 +448,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 /*++++++++++++++++++++++++++++++++++++++
   Find all routes from a specified super-node to any other super-node that follows a certain type of way.
 
-  Results *FindRoutesWay Returns a set of results.
+  Results *FindSuperRoutes Returns a set of results.
 
   NodesX *nodesx The set of nodes to use.
 
@@ -445,22 +461,28 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
   Way *match A template for the type of way that the route must follow.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,node_t start,Way *match)
+static Results *FindSuperRoutes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,node_t start,Way *match)
 {
- Results *results;
- Queue *queue;
+ static Results *results=NULL;
+ static Queue *queue=NULL;
  Result *result1,*result2;
  WayX *wayx;
 
  /* Insert the first node into the queue */
 
- results=NewResultsList(64);
+ if(!results)
+    results=NewResultsList(8);
+ else
+    ResetResultsList(results);
 
- queue=NewQueueList();
+ if(!queue)
+    queue=NewQueueList(8);
+ else
+    ResetQueueList(queue);
 
  result1=InsertResult(results,start,NO_SEGMENT);
 
- InsertInQueue(queue,result1);
+ InsertInQueue(queue,result1,0);
 
  /* Loop across all nodes in the queue */
 
@@ -512,21 +534,19 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
           result2=InsertResult(results,node2,seg2);
           result2->prev=result1;
           result2->score=cumulative_distance;
-          result2->sortby=cumulative_distance;
 
           /* don't route beyond a super-node. */
           if(!IsBitSet(nodesx->super,node2))
-             InsertInQueue(queue,result2);
+             InsertInQueue(queue,result2,cumulative_distance);
          }
        else if(cumulative_distance<result2->score)
          {
           result2->prev=result1;
           result2->score=cumulative_distance;
-          result2->sortby=cumulative_distance;
 
           /* don't route beyond a super-node. */
           if(!IsBitSet(nodesx->super,node2))
-             InsertInQueue(queue,result2);
+             InsertInQueue(queue,result2,cumulative_distance);
          }
 
       endloop:
@@ -535,7 +555,5 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
       }
    }
 
- FreeQueueList(queue);
-
  return(results);
 }
