@@ -66,9 +66,6 @@ static TaggingRuleList *AppendTaggingRule(TaggingRuleList *rules,const char *k,c
 static void AppendTaggingAction(TaggingRuleList *rules,const char *k,const char *v,int action,const char *message);
 static void DeleteTaggingRuleList(TaggingRuleList *rules);
 
-static void ModifyTag(TagList *tags,const char *k,const char *v);
-static void DeleteTag(TagList *tags,const char *k);
-
 static void ApplyRules(TaggingRuleList *rules,TagList *input,TagList *output,const char *match_k,const char *match_v);
 
 
@@ -467,7 +464,7 @@ int ParseXMLTaggingRules(const char *filename)
     return(1);
    }
 
- fd=ReOpenFile(filename);
+ fd=OpenFile(filename);
 
  retval=ParseXML(fd,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_ERRNONAME);
 
@@ -733,6 +730,36 @@ void DeleteTag(TagList *tags,const char *k)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Create a string containing all of the tags formatted as if HTML.
+
+  char *StringifyTag Returns a static pointer to the created string.
+
+  TagList *tags The list of tags to convert.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *StringifyTag(TagList *tags)
+{
+ static char *string=NULL;
+ int i,length=0,used=0;
+
+ for(i=0;i<tags->ntags;i++)
+   {
+    length+=strlen(tags->k[i]);
+    length+=strlen(tags->v[i]);
+
+    length+=16;
+   }
+
+ string=realloc((char*)string,length);
+
+ for(i=0;i<tags->ntags;i++)
+    used+=sprintf(string+used,"<tag k='%s' v='%s'>",tags->k[i],tags->v[i]);
+
+ return(string);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Apply a set of tagging rules to a set of node tags.
 
@@ -864,8 +891,11 @@ static void ApplyRules(TaggingRuleList *rules,TagList *input,TagList *output,con
          }
        else /* if(!k && !v) */
          {
-          for(j=0;j<input->ntags;j++)
-             ApplyRules(rules->rules[i].rulelist,input,output,input->k[j],input->v[j]);
+          if(!input->ntags)
+             ApplyRules(rules->rules[i].rulelist,input,output,"","");
+          else
+             for(j=0;j<input->ntags;j++)
+                ApplyRules(rules->rules[i].rulelist,input,output,input->k[j],input->v[j]);
          }
        break;
 
@@ -927,11 +957,11 @@ static void ApplyRules(TaggingRuleList *rules,TagList *input,TagList *output,con
                }
 
        if(current_list==&NodeRules)
-          logerror("Node %"PRIu64" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",current_id,k,v,rules->rules[i].message);
+          logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",logerror_node(current_id),k,v,rules->rules[i].message);
        if(current_list==&WayRules)
-          logerror("Way %"PRIu64" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",current_id,k,v,rules->rules[i].message);
+          logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",logerror_way(current_id),k,v,rules->rules[i].message);
        if(current_list==&RelationRules)
-          logerror("Relation %"PRIu64" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",current_id,k,v,rules->rules[i].message);
+          logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (in tagging rules); %s.\n",logerror_relation(current_id),k,v,rules->rules[i].message);
       }
    }
 
