@@ -29,6 +29,7 @@
 #include "segments.h"
 #include "ways.h"
 #include "relations.h"
+#include "errorlog.h"
 
 #include "visualiser.h"
 
@@ -68,6 +69,7 @@ static Properties properties=Properties_None;
 /* Local functions */
 
 static void find_all_nodes(Nodes *nodes,callback_t callback);
+
 static void output_junctions(index_t node,double latitude,double longitude);
 static void output_super(index_t node,double latitude,double longitude);
 static void output_oneway(index_t node,double latitude,double longitude);
@@ -154,7 +156,7 @@ static void output_junctions(index_t node,double latitude,double longitude)
  while(segmentp);
 
  if(count!=2 || difference)
-    printf("%.6f %.6f %d\n",radians_to_degrees(latitude),radians_to_degrees(longitude),count);
+    printf("node%"Pindex_t" %.6f %.6f %d\n",node,radians_to_degrees(latitude),radians_to_degrees(longitude),count);
 }
 
 
@@ -216,7 +218,7 @@ static void output_super(index_t node,double latitude,double longitude)
  if(!IsSuperNode(nodep))
     return;
 
- printf("%.6f %.6f n\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
+ printf("node%"Pindex_t" %.6f %.6f\n",node,radians_to_degrees(latitude),radians_to_degrees(longitude));
 
  segmentp=FirstSegment(OSMSegments,nodep,1);
 
@@ -227,10 +229,10 @@ static void output_super(index_t node,double latitude,double longitude)
        index_t othernode=OtherNode(segmentp,node);
        double lat,lon;
 
-       GetLatLong(OSMNodes,othernode,&lat,&lon);
+       GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
        if(node>othernode || (lat<LatMin || lat>LatMax || lon<LonMin || lon>LonMax))
-          printf("%.6f %.6f s\n",radians_to_degrees(lat),radians_to_degrees(lon));
+          printf("segment%"Pindex_t" %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(lat),radians_to_degrees(lon));
       }
 
     segmentp=NextSegment(OSMSegments,segmentp,node);
@@ -306,12 +308,12 @@ static void output_oneway(index_t node,double latitude,double longitude)
          {
           double lat,lon;
 
-          GetLatLong(OSMNodes,othernode,&lat,&lon);
+          GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
           if(IsOnewayFrom(segmentp,node))
-             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
           else if(IsOnewayFrom(segmentp,othernode))
-             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(lat),radians_to_degrees(lon),radians_to_degrees(latitude),radians_to_degrees(longitude));
+             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(lat),radians_to_degrees(lon),radians_to_degrees(latitude),radians_to_degrees(longitude));
          }
       }
 
@@ -396,9 +398,9 @@ static void output_highway(index_t node,double latitude,double longitude)
             {
              double lat,lon;
 
-             GetLatLong(OSMNodes,othernode,&lat,&lon);
+             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
-             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
             }
          }
       }
@@ -484,9 +486,9 @@ static void output_transport(index_t node,double latitude,double longitude)
             {
              double lat,lon;
 
-             GetLatLong(OSMNodes,othernode,&lat,&lon);
+             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
-             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
             }
          }
       }
@@ -556,7 +558,7 @@ static void output_barrier(index_t node,double latitude,double longitude)
  Node *nodep=LookupNode(OSMNodes,node,1);
 
  if(!(nodep->allow&transports))
-    printf("%.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
+    printf("node%"Pindex_t" %.6f %.6f\n",node,radians_to_degrees(latitude),radians_to_degrees(longitude));
 }
 
 
@@ -633,14 +635,16 @@ static void output_turnrestriction(index_t node,double latitude,double longitude
     to_segmentp  =LookupSegment(OSMSegments,relation->to  ,2);
 
     from_node=OtherNode(from_segmentp,node);
-    to_node=OtherNode(to_segmentp,node);
+    to_node  =OtherNode(to_segmentp  ,node);
 
-    GetLatLong(OSMNodes,from_node,&from_lat,&from_lon);
-    GetLatLong(OSMNodes,to_node,&to_lat,&to_lon);
+    GetLatLong(OSMNodes,from_node,NULL,&from_lat,&from_lon);
+    GetLatLong(OSMNodes,to_node  ,NULL,&to_lat  ,&to_lon);
 
-    printf("%.6f %.6f %.6f %.6f %.6f %.6f\n",radians_to_degrees(from_lat),radians_to_degrees(from_lon),
-                                             radians_to_degrees(latitude),radians_to_degrees(longitude),
-                                             radians_to_degrees(to_lat),radians_to_degrees(to_lon));
+    printf("turn-relation%"Pindex_t" %.6f %.6f %.6f %.6f %.6f %.6f\n",
+           turnrelation,
+           radians_to_degrees(from_lat),radians_to_degrees(from_lon),
+           radians_to_degrees(latitude),radians_to_degrees(longitude),
+           radians_to_degrees(to_lat),radians_to_degrees(to_lon));
 
     turnrelation=FindNextTurnRelation1(OSMRelations,turnrelation);
    }
@@ -872,6 +876,7 @@ static void output_limits(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
  Segment *segmentp,segmentps[MAX_SEG_PER_NODE];
+ index_t segments[MAX_SEG_PER_NODE];
  int limits[MAX_SEG_PER_NODE];
  int count=0;
  int i,j,same=0;
@@ -885,6 +890,7 @@ static void output_limits(index_t node,double latitude,double longitude)
        Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
        segmentps[count]=*segmentp;
+       segments [count]=IndexSegment(OSMSegments,segmentp);
 
        switch(limit_type)
          {
@@ -920,7 +926,7 @@ static void output_limits(index_t node,double latitude,double longitude)
 
  /* Display the interesting limits */
 
- printf("%.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
+ printf("node%"Pindex_t" %.6f %.6f\n",node,radians_to_degrees(latitude),radians_to_degrees(longitude));
 
  for(i=0;i<count;i++)
    {
@@ -933,24 +939,24 @@ static void output_limits(index_t node,double latitude,double longitude)
       {
        double lat,lon;
 
-       GetLatLong(OSMNodes,OtherNode(&segmentps[i],node),&lat,&lon);
+       GetLatLong(OSMNodes,OtherNode(&segmentps[i],node),NULL,&lat,&lon);
 
        switch(limit_type)
          {
          case SPEED_LIMIT:
-          printf("%.6f %.6f %d\n",radians_to_degrees(lat),radians_to_degrees(lon),speed_to_kph(limits[i]));
+          printf("segment%"Pindex_t" %.6f %.6f %d\n",segments[i],radians_to_degrees(lat),radians_to_degrees(lon),speed_to_kph(limits[i]));
           break;
          case WEIGHT_LIMIT:
-          printf("%.6f %.6f %.1f\n",radians_to_degrees(lat),radians_to_degrees(lon),weight_to_tonnes(limits[i]));
+          printf("segment%"Pindex_t" %.6f %.6f %.1f\n",segments[i],radians_to_degrees(lat),radians_to_degrees(lon),weight_to_tonnes(limits[i]));
           break;
          case HEIGHT_LIMIT:
-          printf("%.6f %.6f %.1f\n",radians_to_degrees(lat),radians_to_degrees(lon),height_to_metres(limits[i]));
+          printf("segment%"Pindex_t" %.6f %.6f %.1f\n",segments[i],radians_to_degrees(lat),radians_to_degrees(lon),height_to_metres(limits[i]));
           break;
          case WIDTH_LIMIT:
-          printf("%.6f %.6f %.1f\n",radians_to_degrees(lat),radians_to_degrees(lon),width_to_metres(limits[i]));
+          printf("segment%"Pindex_t" %.6f %.6f %.1f\n",segments[i],radians_to_degrees(lat),radians_to_degrees(lon),width_to_metres(limits[i]));
           break;
          case LENGTH_LIMIT:
-          printf("%.6f %.6f %.1f\n",radians_to_degrees(lat),radians_to_degrees(lon),length_to_metres(limits[i]));
+          printf("segment%"Pindex_t" %.6f %.6f %.1f\n",segments[i],radians_to_degrees(lat),radians_to_degrees(lon),length_to_metres(limits[i]));
           break;
          }
       }
@@ -1033,9 +1039,9 @@ static void output_property(index_t node,double latitude,double longitude)
             {
              double lat,lon;
 
-             GetLatLong(OSMNodes,othernode,&lat,&lon);
+             GetLatLong(OSMNodes,othernode,NULL,&lat,&lon);
 
-             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+             printf("segment%"Pindex_t" %.6f %.6f %.6f %.6f\n",IndexSegment(OSMSegments,segmentp),radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
             }
          }
       }
@@ -1088,3 +1094,56 @@ static void find_all_nodes(Nodes *nodes,callback_t callback)
          }
       }
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Output the data for error logs within the region.
+
+  ErrorLogs *errorlogs The set of error logs to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void OutputErrorLog(ErrorLogs *errorlogs,double latmin,double latmax,double lonmin,double lonmax)
+{
+ ll_bin_t latminbin=latlong_to_bin(radians_to_latlong(latmin))-errorlogs->file.latzero;
+ ll_bin_t latmaxbin=latlong_to_bin(radians_to_latlong(latmax))-errorlogs->file.latzero;
+ ll_bin_t lonminbin=latlong_to_bin(radians_to_latlong(lonmin))-errorlogs->file.lonzero;
+ ll_bin_t lonmaxbin=latlong_to_bin(radians_to_latlong(lonmax))-errorlogs->file.lonzero;
+ ll_bin_t latb,lonb;
+ index_t i,index1,index2;
+
+ /* Loop through all of the error logs. */
+
+ for(latb=latminbin;latb<=latmaxbin;latb++)
+    for(lonb=lonminbin;lonb<=lonmaxbin;lonb++)
+      {
+       ll_bin2_t llbin=lonb*errorlogs->file.latbins+latb;
+
+       if(llbin<0 || llbin>(errorlogs->file.latbins*errorlogs->file.lonbins))
+          continue;
+
+       index1=LookupErrorLogOffset(errorlogs,llbin);
+       index2=LookupErrorLogOffset(errorlogs,llbin+1);
+
+       if(index2>errorlogs->file.number_geo)
+          index2=errorlogs->file.number_geo;
+
+       for(i=index1;i<index2;i++)
+         {
+          ErrorLog *errorlogp=LookupErrorLog(errorlogs,i,1);
+
+          double lat=latlong_to_radians(bin_to_latlong(errorlogs->file.latzero+latb)+off_to_latlong(errorlogp->latoffset));
+          double lon=latlong_to_radians(bin_to_latlong(errorlogs->file.lonzero+lonb)+off_to_latlong(errorlogp->lonoffset));
+
+          if(lat>latmin && lat<latmax && lon>lonmin && lon<lonmax)
+             printf("errorlog%"Pindex_t" %.6f %.6f\n",i,radians_to_degrees(lat),radians_to_degrees(lon));
+         }
+      }
+}
