@@ -613,7 +613,7 @@ function updateURL(element)     // called from router.html
     element.href="visualiser.html" + "?" + buildMapArguments();
 
  if(element.id == "edit_url")
-    element.href="http://www.openstreetmap.org/edit" + "?" + buildMapArguments();
+    element.href=mapprops.editurl + "?" + buildMapArguments();
 
  if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
     element.href="router.html" + "." + RegExp.$1 + "?" + buildURLArguments(false) + ";" + buildMapArguments();
@@ -742,7 +742,7 @@ function map_init()             // called from router.html
 
  // Add a vectors layer
 
- layerVectors = new OpenLayers.Layer.Vector("Markers");
+ layerVectors = new OpenLayers.Layer.Vector("Markers",{displayInLayerSwitcher: false});
  map.addLayer(layerVectors);
 
  // A set of markers
@@ -815,6 +815,16 @@ function map_init()             // called from router.html
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
    }
+
+ // Unhide editing URL if variable set
+
+ if(mapprops.editurl != undefined && mapprops.editurl != "")
+   {
+    edit_url=document.getElementById("edit_url");
+
+    edit_url.style.display="";
+    edit_url.href=mapprops.editurl;
+   }
 }
 
 
