@@ -37,47 +37,11 @@ var data_types=[
                 "height",
                 "width",
                 "length",
-                "property"
+                "property",
+                "errorlogs"
                ];
 
 
-//
-// Junction styles
-//
-
-var junction_colours={
-                      0: "#FFFFFF",
-                      1: "#FF0000",
-                      2: "#FFFF00",
-                      3: "#00FF00",
-                      4: "#8B4513",
-                      5: "#00BFFF",
-                      6: "#FF69B4",
-                      7: "#000000",
-                      8: "#000000",
-                      9: "#000000"
-                     };
-
-var junction_styles={};
-
-
-//
-// Super styles
-//
-
-var super_node_style,super_segment_style;
-
-
-//
-// Oneway and turn restriction styles
-//
-
-var hex={0: "00", 1: "11",  2: "22",  3: "33",  4: "44",  5: "55",  6: "66",  7: "77",
-         8: "88", 9: "99", 10: "AA", 11: "BB", 12: "CC", 13: "DD", 14: "EE", 15: "FF"};
-
-var turn_restriction_style;
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////// Initialisation /////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -119,10 +83,11 @@ if(location.search.length>1)
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMap=[], layerVectors, layerBoxes;
+var layerMap=[], layerHighlights, layerVectors, layerBoxes;
 var epsg4326, epsg900913;
 
 var box;
+var select;
 
 // 
 // Initialise the 'map' object
@@ -229,22 +194,27 @@ function map_init()             // called from visualiser.html
   return OpenLayers.String.format(url, xyz);
  }
 
- // Add a vectors layer
+ // Add two vectors layers (one for highlights that display behind the vectors)
  
- layerVectors = new OpenLayers.Layer.Vector("Markers");
+ layerHighlights = new OpenLayers.Layer.Vector("Highlights",{displayInLayerSwitcher: false});
+ map.addLayer(layerHighlights);
+
+ layerVectors = new OpenLayers.Layer.Vector("Markers",{displayInLayerSwitcher: false});
  map.addLayer(layerVectors);
 
- for(var colour in junction_colours)
-    junction_styles[colour]=new OpenLayers.Style({},{stroke: false, pointRadius: 2,fillColor: junction_colours[colour]});
+ // Handle feature selection and popup
 
- super_node_style   =new OpenLayers.Style({},{stroke: false, pointRadius: 3,fillColor  : "#FF0000"});
- super_segment_style=new OpenLayers.Style({},{fill: false  , strokeWidth: 2,strokeColor: "#FF0000"});
+ select = new OpenLayers.Control.SelectFeature(layerVectors,
+                                               {onSelect: selectFeature, onUnselect: unselectFeature});
 
- turn_restriction_style=new OpenLayers.Style({},{fill: false, strokeWidth: 2,strokeColor: "#FF0000"});
+ map.addControl(select);
+ select.activate();
+
+ createPopup();
 
  // Add a boxes layer
 
- layerBoxes = new OpenLayers.Layer.Boxes("Boundary");
+ layerBoxes = new OpenLayers.Layer.Boxes("Boundary",{displayInLayerSwitcher: false});
  map.addLayer(layerBoxes);
 
  box=null;
@@ -272,6 +242,16 @@ function map_init()             // called from visualiser.html
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
    }
+
+ // Unhide editing URL if variable set
+
+ if(mapprops.editurl != undefined && mapprops.editurl != "")
+   {
+    edit_url=document.getElementById("edit_url");
+
+    edit_url.style.display="";
+    edit_url.href=mapprops.editurl;
+   }
 }
 
 
@@ -327,13 +307,142 @@ function updateURL(element)     // called from visualiser.html
     element.href="router.html" + "?" + buildMapArguments();
 
  if(element.id == "edit_url")
-    element.href="http://www.openstreetmap.org/edit" + "?" + buildMapArguments();
+    element.href=mapprops.editurl + "?" + buildMapArguments();
 
  if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
     element.href="visualiser.html" + "." + RegExp.$1 + "?" + buildMapArguments();
 }
 
 
+////////////////////////////////////////////////////////////////////////////////
+///////////////////////// Popup and selection handling /////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+
+var popup=null;
+
+//
+// Create a popup - not using OpenLayers because want it fixed on screen not fixed on map.
+//
+
+function createPopup()
+{
+ popup=document.createElement('div');
+
+ popup.className = "popup";
+
+ popup.innerHTML = "<span></span>";
+
+ popup.style.display = "none";
+
+ popup.style.position = "fixed";
+ popup.style.top = "-4000px";
+ popup.style.left = "-4000px";
+ popup.style.zIndex = "100";
+
+ popup.style.padding = "5px";
+
+ popup.style.opacity=0.85;
+ popup.style.backgroundColor="#C0C0C0";
+ popup.style.border="4px solid #404040";
+
+ document.body.appendChild(popup);
+}
+
+
+//
+// Draw a popup - not using OpenLayers because want it fixed on screen not fixed on map.
+//
+
+function drawPopup(html)
+{
+ if(html==null)
+   {
+    popup.style.display="none";
+    return;
+   }
+
+ if(popup.style.display=="none")
+   {
+    var map_div=document.getElementById("map");
+
+    popup.style.left  =map_div.offsetParent.offsetLeft+map_div.offsetLeft+60 + "px";
+    popup.style.top   =                                map_div.offsetTop +30 + "px";
+    popup.style.width =map_div.clientWidth-100 + "px";
+
+    popup.style.display="";
+   }
+
+ popup.innerHTML=html;
+}
+
+
+//
+// Select a feature
+//
+
+function selectFeature(feature)
+{
+ if(feature.attributes.dump)
+    OpenLayers.Request.GET({url: "visualiser.cgi?dump=" + feature.attributes.dump, success: runDumpSuccess});
+
+ layerHighlights.destroyFeatures();
+
+ highlight_style = new OpenLayers.Style({},{strokeColor: "#F0F000",strokeWidth: 8,
+                                            fillColor: "#F0F000",pointRadius: 4});
+
+ highlight = new OpenLayers.Feature.Vector(feature.geometry.clone(),{},highlight_style);
+
+ layerHighlights.addFeatures([highlight]);
+}
+
+
+//
+// Un-select a feature
+//
+
+function unselectFeature(feature)
+{
+ layerHighlights.destroyFeatures();
+
+ drawPopup(null);
+}
+
+
+//
+// Display the dump data
+//
+
+function runDumpSuccess(response)
+{
+ var string=response.responseText;
+
+ if(mapprops.editurl != undefined && mapprops.editurl != "")
+   {
+    var types=["node", "way", "relation"];
+    var Types=["Node", "Way", "Relation"];
+
+    for(var t in types)
+      {
+       var Type=Types[t];
+       var type=types[t];
+
+       var regexp=RegExp(Type + " [0-9]+");
+
+       while((match=string.match(regexp)) != null)
+         {
+          match=String(match);
+
+          var id=match.slice(1+type.length,match.length);
+
+          string=string.replace(regexp,Type + " <a href='" + mapprops.browseurl + "/" + type + "/" + id + "' target='" + type + id + "'>" + id + "</a>");
+         }
+      }
+   }
+
+ drawPopup(string.split("\n").join("<br>"));
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////// Server handling ////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -405,7 +514,12 @@ function displayData(datatype)  // called from visualiser.html
 
  // Delete the old data
 
+ unselectFeature();
+
+ select.deactivate();
+
  layerVectors.destroyFeatures();
+ layerHighlights.destroyFeatures();
 
  if(box != null)
     layerBoxes.removeMarker(box);
@@ -441,7 +555,7 @@ function displayData(datatype)  // called from visualiser.html
     OpenLayers.Request.GET({url: url, success: runJunctionsSuccess, failure: runFailure});
     break;
    case 'super':
-    OpenLayers.Request.GET({url: url, success: runSuperSuccess, faliure: runFailure});
+    OpenLayers.Request.GET({url: url, success: runSuperSuccess, failure: runFailure});
     break;
    case 'oneway':
     OpenLayers.Request.GET({url: url, success: runOnewaySuccess, failure: runFailure});
@@ -452,7 +566,7 @@ function displayData(datatype)  // called from visualiser.html
        if(highways[h].checked)
           highway=highways[h].value;
     url+="-" + highway;
-    OpenLayers.Request.GET({url: url, success: runHighwaySuccess, falure: runFailure});
+    OpenLayers.Request.GET({url: url, success: runHighwaySuccess, failure: runFailure});
     break;
    case 'transport':
     var transports=document.forms["transports"].elements["transport"];
@@ -486,7 +600,10 @@ function displayData(datatype)  // called from visualiser.html
        if(properties[p].checked)
           property=properties[p].value;
     url+="-" + property;
-    OpenLayers.Request.GET({url: url, success: runPropertySuccess, falure: runFailure});
+    OpenLayers.Request.GET({url: url, success: runPropertySuccess, failure: runFailure});
+    break;
+   case 'errorlogs':
+    OpenLayers.Request.GET({url: url, success: runErrorlogSuccess, failure: runFailure});
     break;
    }
 }
@@ -500,6 +617,26 @@ function runJunctionsSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var junction_colours={
+                       0: "#FFFFFF",
+                       1: "#FF0000",
+                       2: "#FFFF00",
+                       3: "#00FF00",
+                       4: "#8B4513",
+                       5: "#00BFFF",
+                       6: "#FF69B4",
+                       7: "#000000",
+                       8: "#000000",
+                       9: "#000000"
+                      };
+
+ var styles={};
+
+ for(var colour in junction_colours)
+    styles[colour]=new OpenLayers.Style({},{stroke: false,
+                                            pointRadius: 2,fillColor: junction_colours[colour],
+                                            cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -521,18 +658,21 @@ function runJunctionsSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat=words[0];
-       var lon=words[1];
-       var count=words[2];
+       var dump=words[0];
+       var lat=words[1];
+       var lon=words[2];
+       var count=words[3];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
        var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
 
-       features.push(new OpenLayers.Feature.Vector(point,{},junction_styles[count]));
+       features.push(new OpenLayers.Feature.Vector(point,{dump: dump},styles[count]));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","junctions",lines.length-2);
@@ -547,6 +687,14 @@ function runSuperSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var node_style = new OpenLayers.Style({},{stroke: false,
+                                           pointRadius: 4,fillColor: "#FF0000",
+                                           cursor: "pointer"});
+
+ var segment_style = new OpenLayers.Style({},{fill: false,
+                                              strokeWidth: 2,strokeColor: "#FF0000",
+                                              cursor: "pointer"});
+
  var features=[];
 
  var nodepoint;
@@ -570,29 +718,31 @@ function runSuperSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat=words[0];
-       var lon=words[1];
-       var type=words[2];
+       var dump=words[0];
+       var lat=words[1];
+       var lon=words[2];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
        var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
 
-       if(type == "n")
+       if(dump.charAt(0) == "n")
          {
           nodepoint=point;
 
-          features.push(new OpenLayers.Feature.Vector(point,{},super_node_style));
+          features.push(new OpenLayers.Feature.Vector(point,{dump: dump},node_style));
          }
        else
          {
           var segment = new OpenLayers.Geometry.LineString([nodepoint,point]);
 
-          features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+          features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},segment_style));
          }
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","super",lines.length-2);
@@ -605,6 +755,9 @@ function runSuperSuccess(response)
 
 function runOnewaySuccess(response)
 {
+ var hex={0: "00", 1: "11",  2: "22",  3: "33",  4: "44",  5: "55",  6: "66",  7: "77",
+          8: "88", 9: "99", 10: "AA", 11: "BB", 12: "CC", 13: "DD", 14: "EE", 15: "FF"};
+
  var lines=response.responseText.split('\n');
 
  var features=[];
@@ -628,10 +781,11 @@ function runOnewaySuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat1=words[0];
-       var lon1=words[1];
-       var lat2=words[2];
-       var lon2=words[3];
+       var dump=words[0];
+       var lat1=words[1];
+       var lon1=words[2];
+       var lat2=words[3];
+       var lon2=words[4];
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -654,12 +808,14 @@ function runOnewaySuccess(response)
        var b=Math.round(7.5+7.9*Math.cos(ang-2.0943951));
        var colour = "#" + hex[r] + hex[g] + hex[b];
 
-       var style=new OpenLayers.Style({},{strokeWidth: 2,strokeColor: colour});
+       var style=new OpenLayers.Style({},{strokeWidth: 2,strokeColor: colour, cursor: "pointer"});
 
-       features.push(new OpenLayers.Feature.Vector(segment,{},style));
+       features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","oneway",lines.length-2);
@@ -674,6 +830,10 @@ function runHighwaySuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var style = new OpenLayers.Style({},{fill: false,
+                                      strokeWidth: 2,strokeColor: "#FF0000",
+                                      cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -695,10 +855,11 @@ function runHighwaySuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat1=words[0];
-       var lon1=words[1];
-       var lat2=words[2];
-       var lon2=words[3];
+       var dump=words[0];
+       var lat1=words[1];
+       var lon1=words[2];
+       var lat2=words[3];
+       var lon2=words[4];
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -708,10 +869,12 @@ function runHighwaySuccess(response)
 
        var segment = new OpenLayers.Geometry.LineString([point1,point2]);
 
-       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+       features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","highway",lines.length-2);
@@ -726,6 +889,10 @@ function runTransportSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var style = new OpenLayers.Style({},{fill: false,
+                                      strokeWidth: 2,strokeColor: "#FF0000",
+                                      cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -747,10 +914,11 @@ function runTransportSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat1=words[0];
-       var lon1=words[1];
-       var lat2=words[2];
-       var lon2=words[3];
+       var dump=words[0];
+       var lat1=words[1];
+       var lon1=words[2];
+       var lat2=words[3];
+       var lon2=words[4];
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -760,10 +928,12 @@ function runTransportSuccess(response)
 
        var segment = new OpenLayers.Geometry.LineString([point1,point2]);
 
-       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+       features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","transport",lines.length-2);
@@ -778,6 +948,10 @@ function runBarrierSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var style = new OpenLayers.Style({},{stroke: false,
+                                      pointRadius: 3,fillColor: "#FF0000",
+                                      cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -799,17 +973,20 @@ function runBarrierSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat=words[0];
-       var lon=words[1];
+       var dump=words[0];
+       var lat=words[1];
+       var lon=words[2];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
        var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
 
-       features.push(new OpenLayers.Feature.Vector(point,{},super_node_style));
+       features.push(new OpenLayers.Feature.Vector(point,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","barrier",lines.length-2);
@@ -824,6 +1001,10 @@ function runTurnsSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var style = new OpenLayers.Style({},{fill: false,
+                                      strokeWidth: 2,strokeColor: "#FF0000",
+                                      cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -845,12 +1026,13 @@ function runTurnsSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat1=words[0];
-       var lon1=words[1];
-       var lat2=words[2];
-       var lon2=words[3];
-       var lat3=words[4];
-       var lon3=words[5];
+       var dump=words[0];
+       var lat1=words[1];
+       var lon1=words[2];
+       var lat2=words[3];
+       var lon2=words[4];
+       var lat3=words[5];
+       var lon3=words[6];
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -862,10 +1044,12 @@ function runTurnsSuccess(response)
 
        var segments = new OpenLayers.Geometry.LineString([point1,point2,point3]);
 
-       features.push(new OpenLayers.Feature.Vector(segments,{},turn_restriction_style));
+       features.push(new OpenLayers.Feature.Vector(segments,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","turns",lines.length-2);
@@ -880,8 +1064,17 @@ function runLimitSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var node_style = new OpenLayers.Style({},{stroke: false,
+                                           pointRadius: 3,fillColor: "#FF0000",
+                                           cursor: "pointer"});
+
+ var segment_style = new OpenLayers.Style({},{fill: false,
+                                              strokeWidth: 2,strokeColor: "#FF0000",
+                                              cursor: "pointer"});
+
  var features=[];
 
+ var nodepoint;
  var nodelonlat;
 
  for(var line=0;line<lines.length;line++)
@@ -903,9 +1096,10 @@ function runLimitSuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat=words[0];
-       var lon=words[1];
-       var number=words[2];
+       var dump=words[0];
+       var lat=words[1];
+       var lon=words[2];
+       var number=words[3];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
@@ -914,18 +1108,25 @@ function runLimitSuccess(response)
           var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
 
           nodelonlat=lonlat;
+          nodepoint = point;
 
-          features.push(new OpenLayers.Feature.Vector(point,{},junction_styles[1]));
+          features.push(new OpenLayers.Feature.Vector(point,{dump: dump},node_style));
          }
        else
          {
+          var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
+
+          var segment = new OpenLayers.Geometry.LineString([nodepoint,point]);
+
+          features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},segment_style));
+
           var dlat = lonlat.lat-nodelonlat.lat;
           var dlon = lonlat.lon-nodelonlat.lon;
-          var dist = Math.sqrt(dlat*dlat+dlon*dlon)/60;
+          var dist = Math.sqrt(dlat*dlat+dlon*dlon)/120;
 
           var point = new OpenLayers.Geometry.Point(nodelonlat.lon+dlon/dist,nodelonlat.lat+dlat/dist);
 
-          features.push(new OpenLayers.Feature.Vector(point,{},
+          features.push(new OpenLayers.Feature.Vector(point,{dump: dump},
                                                       new OpenLayers.Style({},{externalGraphic: 'icons/limit-' + number + '.png',
                                                                                graphicYOffset: -9,
                                                                                graphicWidth: 19,
@@ -934,6 +1135,8 @@ function runLimitSuccess(response)
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","limit",lines.length-2);
@@ -948,6 +1151,10 @@ function runPropertySuccess(response)
 {
  var lines=response.responseText.split('\n');
 
+ var style = new OpenLayers.Style({},{fill: false,
+                                      strokeWidth: 2,strokeColor: "#FF0000",
+                                      cursor: "pointer"});
+
  var features=[];
 
  for(var line=0;line<lines.length;line++)
@@ -969,10 +1176,11 @@ function runPropertySuccess(response)
       }
     else if(words[0] != "")
       {
-       var lat1=words[0];
-       var lon1=words[1];
-       var lat2=words[2];
-       var lon2=words[3];
+       var dump=words[0];
+       var lat1=words[1];
+       var lon1=words[2];
+       var lat2=words[3];
+       var lon2=words[4];
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -982,21 +1190,76 @@ function runPropertySuccess(response)
 
        var segment = new OpenLayers.Geometry.LineString([point1,point2]);
 
-       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+       features.push(new OpenLayers.Feature.Vector(segment,{dump: dump},style));
       }
    }
 
+ select.activate();
+
  layerVectors.addFeatures(features);
 
  displayStatus("data","property",lines.length-2);
 }
 
 
+//
+// Success in getting the error log data
+//
+
+function runErrorlogSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var style = new OpenLayers.Style({},{stroke: false,
+                                      pointRadius: 3,fillColor: "#FF0000",
+                                      cursor: "pointer"});
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var dump=words[0];
+       var lat=words[1];
+       var lon=words[2];
+
+       var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
+
+       var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
+
+       features.push(new OpenLayers.Feature.Vector(point,{dump: dump},style));
+      }
+   }
+
+ select.activate();
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","errorlogs",lines.length-2);
+}
+
+
 //
 // Failure in getting data.
 //
 
 function runFailure(response)
 {
- displayStatus("error");
+ displayStatus("failed");
 }
